library(ggplot2)
library(showtext)
library(tidyr)
library(gridExtra)
library(ggforce)
library(scales)
library(ggpubr)

# Set working directory
setwd("")

font_add("Jost", regular = "Scripts/Jost/Jost-400-Book.ttf")
showtext_auto() # turn on showtext auto mode
theme_set(theme_classic(base_family = "Jost")) # set Jost as the base font family

# This script is used to create Figure 1
# It shows stacked bars per electricity source, showing the median contribution of process steps to total production emissions.
# Split by configurations for intermittent renewables, and showing configuration-independent electricity sources separately.

# Read emissions data for green hydrogen production for the three configurations
# Grid-connected: power export configuration:
dfexport <- read.csv("Derived_data/database_results.csv")
# Off-grid: curtailment configuration:
dfcurt <- read.csv("Derived_data/database_results_curt0.5.csv")
# Grid-connected: power import (hybrid) configuration:
dfimport <- read.csv("Derived_data/database_results_hybrid1.csv")

#specify the production configuration
dfexport$scenario <- "Grid-connected: power export"
dfcurt$scenario <- "Off-grid: curtailment"
dfimport$scenario <- "Grid-connected: power import"

# Select only the relevant electricity sources, i.e. those that are changing between the configurations
dfexport <- subset(dfexport, electricity.source == "Onshore wind" | electricity.source == "Offshore wind" | electricity.source == "Solar PV" | electricity.source == "Renewable mix" )
dfcurt <- subset(dfcurt, electricity.source == "Onshore wind" | electricity.source == "Offshore wind" | electricity.source == "Solar PV" | electricity.source == "Renewable mix" )
dfimport <- subset(dfimport, electricity.source == "Onshore wind" | electricity.source == "Offshore wind" | electricity.source == "Solar PV" | electricity.source == "Renewable mix" )

# For visualisation purpose, the emissions from KOH are included in the electrolyser infrastructure emissions
dfexport$EF_electrolyser_infra <- dfexport$EF_electrolyser + dfexport$EF_electrolysis_KOH

# Set 'zero' values to NA in case of electrolysis_heat and battery, because a zero only means that these are not applicable in that case
dfexport$EF_electrolysis_heat <- ifelse(dfexport$EF_electrolysis_heat == 0, NA, dfexport$EF_electrolysis_heat)
dfexport$EF_battery <- ifelse(dfexport$EF_battery == 0, NA, dfexport$EF_battery)

# Select the relevant columns from the dfexport
dfexport_long <- gather(dfexport, key = "variable", value = "value", 
                        EF_water, EF_electrolysis_electricity, EF_electrolysis_heat,
                        EF_electrolyser_infra, EF_battery)

# To go from emissions expressed as gCO2-e to kgCO2-e 
dfexport_long$value <- dfexport_long$value/1000

# Rename the selected columns for each of the process steps
dfexport_long$variable <- ifelse(dfexport_long$variable == "EF_water", "Water desalination",
                                 ifelse(dfexport_long$variable == "EF_electrolysis_electricity", "Electricity",
                                        ifelse(dfexport_long$variable == "EF_electrolysis_heat", "Heat",
                                               ifelse(dfexport_long$variable == "EF_electrolyser_infra", "Electrolyser",
                                                      ifelse(dfexport_long$variable == "EF_battery", "Battery", NA)))))
# Set the order of the steps
custom_order <- c("Electricity", "Heat",  "Electrolyser", "Water desalination", "Battery")

dfexport_long$variable <- factor(dfexport_long$variable, levels = custom_order)


# Electricity order from low to high footprint based on base case production emissions
electricity_order <- c("Offshore wind","Onshore wind", "Renewable mix", "Solar PV")

# Figure for the grid-connected: power export configuration
contributions_production_export <- ggplot(dfexport_long, aes(x = factor(electricity.source, levels = electricity_order), y = value, fill = variable)) +
  stat_summary(geom = "bar", fun = median, position = "stack", color = "black", linewidth = 0.05, width = 0.8) +
  scale_fill_manual(name = "Production step",
                    values = c("Water desalination" = "#a6dba0", "Electrolyser" = "#f7f7f7", "Electricity" = "#7b3294", 
                               "Heat" = "#c2a5cf", "Battery" = "#008837"),
                    breaks = c("Electricity", "Heat", "Electrolyser", "Water desalination", "Battery"),
                    labels = c("Electrolyser use: electricity", "Electrolyser use: heat",  "Electrolyser production", "Water desalination","Battery production")) +
  scale_y_continuous(breaks = seq(0, 100, by = 2), expand = c(0, 0), limits = c(0, NA)) +
  coord_cartesian(ylim = c(0, 22)) + 
  ylab("E") + 
  xlab("")+
  theme_classic() + 
  theme(text = element_text(family = "Jost", size = 32, color = "black"),
        legend.text = element_text(size = 28),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks.y = element_line(linewidth = 0.2),
        axis.ticks.x = element_line(linewidth = 0),
        axis.text.x = element_text(angle = 50, hjust = 1))

contributions_production_export

# Calculate the median values for each contribution
median_values <- aggregate(value ~ variable, data = dfexport_long, FUN = median)

ggsave("Figures/Figure1B.tiff", contributions_production_export, width = 9, height = 15, units = "cm", dpi = 300)


############################
### For grid-connected: power import configuration (hybrid)

# For visualisation purpose, the emissions from KOH are included in the electrolyser infrastructure emissions
dfimport$EF_electrolyser_infra <- dfimport$EF_electrolyser + dfimport$EF_electrolysis_KOH

# Set 'zero' values to NA in case of electrolysis_heat and battery, because a zero only means that these are not applicable in that case
dfimport$EF_electrolysis_heat <- ifelse(dfimport$EF_electrolysis_heat == 0, NA, dfimport$EF_electrolysis_heat)
dfimport$EF_battery <- ifelse(dfimport$EF_battery == 0, NA, dfimport$EF_battery)


# Select the relevant columns from the dfimport
dfimport_long <- gather(dfimport, key = "variable", value = "value", 
                        EF_water, EF_electrolysis_electricity, EF_electrolysis_heat,
                        EF_electrolyser_infra, EF_battery)

# To go from g to kg emissions
dfimport_long$value <- dfimport_long$value/1000

# Rename the selected columns for each of the process steps
dfimport_long$variable <- ifelse(dfimport_long$variable == "EF_water", "Water desalination",
                                 ifelse(dfimport_long$variable == "EF_electrolysis_electricity", "Electricity",
                                        ifelse(dfimport_long$variable == "EF_electrolysis_heat", "Heat",
                                               ifelse(dfimport_long$variable == "EF_electrolyser_infra", "Electrolyser",
                                                      ifelse(dfimport_long$variable == "EF_battery", "Battery", NA)))))
# Set the order of the steps
custom_order <- c("Electricity", "Heat", "Electrolyser", "Water desalination",  "Battery")

dfimport_long$variable <- factor(dfimport_long$variable, levels = custom_order)

# Electricity order from low to high footprint based on base case production emissions
electricity_order <- c("Offshore wind","Onshore wind", "Renewable mix", "Solar PV")

contributions_production_import <- ggplot(dfimport_long, aes(x = factor(electricity.source, levels = electricity_order), y = value, fill = variable)) +
  stat_summary(geom = "bar", fun = median, position = "stack", color = "black", linewidth = 0.05, width = 0.8) +
  scale_fill_manual(name = "Production step",
                    values = c("Water desalination" = "#a6dba0", "Electrolyser" = "#f7f7f7", "Electricity" = "#7b3294", 
                               "Heat" = "#c2a5cf", "Battery" = "#008837"),
                    breaks = c("Electricity", "Heat", "Electrolyser", "Water desalination", "Battery"),
                    labels = c("Electrolyser use: electricity", "Electrolyser use: heat",  "Electrolyser production", "Water desalination","Battery production")) +
  scale_y_continuous(breaks = seq(0, 100, by = 2), expand = c(0, 0), limits = c(0, NA)) +
  coord_cartesian(ylim = c(0, 22)) + 
  ylab("E") + 
  xlab("")+
  theme_classic() + 
  theme(text = element_text(family = "Jost", size = 32, color = "black"),
        legend.text = element_text(size = 28),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks.y = element_line(linewidth = 0.2),
        axis.ticks.x = element_line(linewidth = 0),
        axis.text.x = element_text(angle = 50, hjust = 1))

contributions_production_import

# Calculate the median values for each contribution
median_values <- aggregate(value ~ variable, data = dfimport_long, FUN = median)

ggsave("Figures/Figure1C.tiff", contributions_production_import, width = 9, height = 15, units = "cm", dpi = 300)


#########
# Off-grid: curtailment configuration

# For visualisation purpose, the emissions from KOH are included in the electrolyser infrastructure emissions
dfcurt$EF_electrolyser_infra <- dfcurt$EF_electrolyser + dfcurt$EF_electrolysis_KOH

# Set 'zero' values to NA in case of electrolysis_heat and battery, because a zero only means that these are not applicable in that case
dfcurt$EF_electrolysis_heat <- ifelse(dfcurt$EF_electrolysis_heat == 0, NA, dfcurt$EF_electrolysis_heat)
dfcurt$EF_battery <- ifelse(dfcurt$EF_battery == 0, NA, dfcurt$EF_battery)

# Select the relevant columns from the dfcurt
dfcurt_long <- gather(dfcurt, key = "variable", value = "value", 
                        EF_water, EF_electrolysis_electricity, EF_electrolysis_heat,
                        EF_electrolyser_infra, EF_battery)

# To go from g to kg emissions
dfcurt_long$value <- dfcurt_long$value/1000

# Rename the selected columns for each of the process steps
dfcurt_long$variable <- ifelse(dfcurt_long$variable == "EF_water", "Water desalination",
                                 ifelse(dfcurt_long$variable == "EF_electrolysis_electricity", "Electricity",
                                        ifelse(dfcurt_long$variable == "EF_electrolysis_heat", "Heat",
                                               ifelse(dfcurt_long$variable == "EF_electrolyser_infra", "Electrolyser",
                                                      ifelse(dfcurt_long$variable == "EF_battery", "Battery", NA)))))
# Set the order of the steps
custom_order <- c("Electricity", "Heat",  "Electrolyser","Water desalination",  "Battery")

dfcurt_long$variable <- factor(dfcurt_long$variable, levels = custom_order)


contributions_production_curt <- ggplot(dfcurt_long, aes(x = factor(electricity.source, levels = electricity_order), y = value, fill = variable)) +
  stat_summary(geom = "bar", fun = median, position = "stack", color = "black", linewidth = 0.05, width = 0.8) +
  scale_fill_manual(name = "Production steps",
                    values = c("Water desalination" = "#a6dba0", "Electrolyser" = "#f7f7f7", "Electricity" = "#7b3294", 
                               "Heat" = "#c2a5cf", "Battery" = "#008837"),                  
                    breaks = c("Electricity", "Heat", "Electrolyser", "Water desalination", "Battery"),
                    labels = c("Electrolyser use: electricity", "Electrolyser use: heat",  "Electrolyser production", "Water desalination","Battery production")) +
  scale_y_continuous(breaks = seq(0, 100, by = 2), expand = c(0, 0), limits = c(0, NA)) +
  coord_cartesian(ylim = c(0, 22)) +  
  ylab(expression(paste("Production emissions (kgCO"[2], "-e per kg H"[2], ")"))) + 
  xlab("")+
  theme_classic() + 
  theme(text = element_text(family = "Jost", size = 32, color = "black"),
        legend.text = element_text(size = 28),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks.y = element_line(linewidth = 0.2),
        axis.ticks.x = element_line(linewidth = 0),
        axis.text.x = element_text(angle = 50, hjust = 1))


contributions_production_curt

# Calculate the median values for each contribution
median_values <- aggregate(value ~ variable, data = dfcurt_long, FUN = median)

ggsave("Figures/Figure1A.tiff", contributions_production_curt, width = 9, height = 15, units = "cm", dpi = 300)

#########
# Configuration-independent electricity sources

# read data
dfexport_indep <- read.csv("Derived_data/database_results.csv")

# Select only the relevant electricity sources, i.e. those that are changing between the scenarios.
dfexport_indep <- subset(dfexport_indep, electricity.source == "Excess renewable" | electricity.source == "Hydropower" | electricity.source == "Nuclear" | electricity.source == "Grid" )

dfexport_indep$scenario <- "Configuration-independent"

# For visualisation purpose, the emissions from KOH are included in the electrolyser infrastructure emissions
dfexport_indep$EF_electrolyser_infra <- dfexport_indep$EF_electrolyser + dfexport_indep$EF_electrolysis_KOH

# Set 'zero' values to NA in case of electrolysis_heat and battery, because a zero only means that these are not applicable in that case
dfexport_indep$EF_electrolysis_heat <- ifelse(dfexport_indep$EF_electrolysis_heat == 0, NA, dfexport_indep$EF_electrolysis_heat)
dfexport_indep$EF_battery <- ifelse(dfexport_indep$EF_battery == 0, NA, dfexport_indep$EF_battery)


# Select the relevant columns from the dfexport_indep
dfexport_indep_long <- gather(dfexport_indep, key = "variable", value = "value", 
                        EF_water, EF_electrolysis_electricity, EF_electrolysis_heat,
                        EF_electrolyser_infra, EF_battery)

# To go from g to kg emissions (need to harmonise this with other scripts)
dfexport_indep_long$value <- dfexport_indep_long$value/1000

# Rename the selected columns for each of the process steps
dfexport_indep_long$variable <- ifelse(dfexport_indep_long$variable == "EF_water", "Water desalination",
                                 ifelse(dfexport_indep_long$variable == "EF_electrolysis_electricity", "Electricity",
                                        ifelse(dfexport_indep_long$variable == "EF_electrolysis_heat", "Heat",
                                               ifelse(dfexport_indep_long$variable == "EF_electrolyser_infra", "Electrolyser",
                                                      ifelse(dfexport_indep_long$variable == "EF_battery", "Battery", NA)))))
# Set the order of the steps
custom_order <- c("Electricity", "Heat",  "Electrolyser", "Water desalination", "Battery")

dfexport_indep_long$variable <- factor(dfexport_indep_long$variable, levels = custom_order)


# Electricity order from low to high footprint based on base case production emissions
electricity_order <- c("Nuclear", "Excess renewable", "Hydropower", "Grid")


contributions_production_indep <- ggplot(dfexport_indep_long, aes(x = factor(electricity.source, levels = electricity_order), y = value, fill = variable)) +
  stat_summary(geom = "bar", fun = median, position = "stack", color = "black", linewidth = 0.05, width = 0.8) +
  scale_fill_manual(name = "Production steps",
                    values = c("Water desalination" = "#a6dba0", "Electrolyser" = "#f7f7f7", "Electricity" = "#7b3294", 
                               "Heat" = "#c2a5cf", "Battery" = "#008837"),
                    breaks = c("Electricity", "Heat", "Electrolyser", "Water desalination", "Battery"),
                    labels = c("Electrolyser use: electricity", "Electrolyser use: heat",  "Electrolyser production", "Water desalination","Battery production")) +
  scale_y_continuous(breaks = seq(0, 100, by = 2), expand = c(0, 0), limits = c(0, NA)) +
  coord_cartesian(ylim = c(0, 22)) +   
  ylab("E") +
  xlab("")+
  theme_classic() + 
  theme(text = element_text(family = "Jost", size = 32, color = "black"),
        legend.text = element_text(size = 28),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks.y = element_line(linewidth = 0.2),
        axis.ticks.x = element_line(linewidth = 0),
        axis.text.x = element_text(angle = 50, hjust = 1))


contributions_production_indep

# Calculate the median values for each contribution
median_values <- aggregate(value ~ variable, data = dfexport_indep_long, FUN = median)


ggsave("Figures/Figure1D.tiff", contributions_production_indep, width = 9, height = 15, units = "cm", dpi = 300)

