library(ggplot2)
library(showtext)
library(gridExtra)

# Set working directory
setwd("")

font_add("Jost", regular = "Scripts/Jost/Jost-400-Book.ttf")
showtext_auto() # turn on showtext auto mode
theme_set(theme_classic(base_family = "Jost")) # set Jost as the base font family

# This script is used to create Figure 2
# It creates box plots of production emissions per electricity source.
# Split by configurations for intermittent renewables, and showing configuration-independent electricity sources separately.

# Read emissions data for green hydrogen production for the three configurations
# Grid-connected: power export configuration:
dfexport <- read.csv("Derived_data/database_results.csv")
# Off-grid: curtailment configuration:
dfcurt <- read.csv("Derived_data/database_results_curt0.5.csv")
# Grid-connected: power import (hybrid) configuration:
dfimport <- read.csv("Derived_data/database_results_hybrid1.csv")

#specify the production configuration
dfexport$scenario <- "Grid-connected: power export"
dfcurt$scenario <- "Off-grid: curtailment"
dfimport$scenario <- "Grid-connected: power import"

#the electricity sources independent of configuration 
dfexport_excess <- subset(dfexport, electricity.source == "Excess renewable")
dfexport_hydro <- subset(dfexport, electricity.source == "Hydropower")
dfexport_nuclear <- subset(dfexport, electricity.source == "Nuclear")
dfexport_grid <- subset(dfexport, electricity.source == "Grid")

dfexport_excess$scenario <- "Configuration-independent"
dfexport_hydro$scenario <- "Configuration-independent"
dfexport_nuclear$scenario <- "Configuration-independent"
dfexport_grid$scenario <- "Configuration-independent"

# select and rename relevant columns

dfexport_excess <- dfexport_excess[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_excess) <- c("reference", "electricity.source", "EF", "scenario")

dfexport_hydro <- dfexport_hydro[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_hydro) <- c("reference", "electricity.source", "EF", "scenario")

dfexport_nuclear <- dfexport_nuclear[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_nuclear) <- c("reference", "electricity.source", "EF", "scenario")

dfexport_grid <- dfexport_grid[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_grid) <- c("reference", "electricity.source", "EF", "scenario")


# renewable electricity sources
dfexport_onshore <- subset(dfexport, electricity.source == "Onshore wind")
dfexport_offshore <- subset(dfexport, electricity.source == "Offshore wind")
dfexport_solar <- subset(dfexport, electricity.source == "Solar PV")
dfexport_intermittent <- subset(dfexport, electricity.source == "Renewable mix")


# select and rename relevant columns
dfexport_onshore<- dfexport_onshore[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_onshore) <- c("reference", "electricity.source", "EF", "scenario")

dfexport_offshore <- dfexport_offshore[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_offshore) <- c("reference", "electricity.source", "EF", "scenario")

dfexport_solar <- dfexport_solar[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_solar) <- c("reference", "electricity.source", "EF", "scenario")

dfexport_intermittent <- dfexport_intermittent[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfexport_intermittent) <- c("reference", "electricity.source", "EF", "scenario")


# renewable electricity sources
dfcurt_onshore <- subset(dfcurt, electricity.source == "Onshore wind")
dfcurt_offshore <- subset(dfcurt, electricity.source == "Offshore wind")
dfcurt_solar <- subset(dfcurt, electricity.source == "Solar PV")
dfcurt_intermittent <- subset(dfcurt, electricity.source == "Renewable mix")

dfcurt_onshore <- dfcurt_onshore[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfcurt_onshore) <- c("reference", "electricity.source", "EF", "scenario")

dfcurt_offshore <- dfcurt_offshore[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfcurt_offshore) <- c("reference", "electricity.source", "EF", "scenario")

dfcurt_solar <- dfcurt_solar[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfcurt_solar) <- c("reference", "electricity.source", "EF", "scenario")

dfcurt_intermittent <- dfcurt_intermittent[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfcurt_intermittent) <- c("reference", "electricity.source", "EF", "scenario")


# renewable electricity sources
dfimport_onshore <- subset(dfimport, electricity.source == "Onshore wind")
dfimport_offshore <- subset(dfimport, electricity.source == "Offshore wind")
dfimport_solar <- subset(dfimport, electricity.source == "Solar PV")
dfimport_intermittent <- subset(dfimport, electricity.source == "Renewable mix")


# select and rename relevant columns

dfimport_onshore <- dfimport_onshore[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfimport_onshore) <- c("reference", "electricity.source", "EF", "scenario")

dfimport_offshore <- dfimport_offshore[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfimport_offshore) <- c("reference", "electricity.source", "EF", "scenario")

dfimport_solar <- dfimport_solar[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfimport_solar) <- c("reference", "electricity.source", "EF", "scenario")

dfimport_intermittent <- dfimport_intermittent[,c("reference", "electricity.source", "EF_production", "scenario")]
colnames(dfimport_intermittent) <- c("reference", "electricity.source", "EF", "scenario")


#Combine data
combined_data <- rbind(dfexport_grid, dfexport_excess, dfexport_nuclear, dfexport_hydro, 
                       dfcurt_onshore, dfcurt_offshore, dfcurt_solar, dfcurt_intermittent,
                       dfexport_onshore, dfexport_offshore, dfexport_solar, dfexport_intermittent,
                       dfimport_onshore, dfimport_offshore, dfimport_solar, dfimport_intermittent)

scenario_order <- c("Off-grid: curtailment", "Grid-connected: power export", "Grid-connected: power import", "Configuration-independent")

combined_data$electricity.source <- factor(combined_data$electricity.source, 
                                           levels = c("Grid", "Excess renewable", "Nuclear", "Hydropower",
                                                      "Onshore wind", "Offshore wind", "Solar PV", "Renewable mix"))

electricity_order <- c("Grid", "Excess renewable", "Nuclear", "Hydropower",
                       "Onshore wind", "Offshore wind", "Solar PV", "Renewable mix")


# Create a faceted boxplot
boxplot_prod_elec_facet <- ggplot(combined_data, aes(x = factor(electricity.source, 
                                                                levels = c("Offshore wind", "Onshore wind", "Renewable mix", "Solar PV", "Nuclear", "Hydropower", "Excess renewable", "Grid")), 
                                                     y = EF/1000, fill = electricity.source)) +
  annotate("rect", ymin = 10.5, ymax = 17.1, xmin = -Inf, xmax = Inf, alpha = 0.3, fill="#c5c7c8") +
  annotate("rect", ymin = 2.29, ymax = 8.9, xmin = -Inf, xmax = Inf, alpha = 0.3, fill="#6baed6") +
  geom_hline(aes(yintercept = 4.0, linetype = "US clean"), color = "#183919", linewidth = 0.5) +
  geom_hline(aes(yintercept = 3.0, linetype = "EU taxonomy"), color = "#378139", linewidth = 0.5) +
  geom_hline(aes(yintercept = 2.4, linetype = "UK low carbon"), color = "#74c376", linewidth = 0.5) +
  geom_hline(aes(yintercept = 3.3, linetype = "Blue H2"), color = "#6baed6", linewidth = 0.5) +
  geom_hline(aes(yintercept = 11.5, linetype = "Grey H2"), color = "#c5c7c8", linewidth = 0.5) +
  geom_boxplot(linewidth = 0.1, outlier.size = 0.2, varwidth = F) +
  facet_wrap(factor(scenario, levels = scenario_order) ~ ., nrow = 1, scales = "free_x") +  # Facet by scenario   
  scale_fill_manual(name = "Electricity source",
                    values = c("Onshore wind" = "#b3de69", "Offshore wind" = "#8dd3c7", "Hydropower" = "#80b1d3", "Nuclear" = "#fccde5", 
                               "Excess renewable" = "#fdb462", "Solar PV" = "#ffed6f", "Grid" = "#fc9272", "Renewable mix" = "#d7b2d7"),
                    breaks = c("Offshore wind","Onshore wind", "Renewable mix", "Solar PV","Nuclear", "Hydropower", "Excess renewable", "Grid"),
                    labels = c("Offshore wind","Onshore wind",  "Renewable mix", "Solar photovoltaics","Nuclear", "Hydropower", "Excess renewable", "Grid")) +
  scale_y_continuous(breaks = seq(0, max(combined_data$EF/1000), by = 2), expand = c(0, 0), 
                     limits = c(0, max(combined_data$EF/1000))) +
  scale_x_discrete(labels = NULL, breaks = NULL) +
  xlab("") +
  ylab(expression(paste("Production emissions (kgCO"[2], "-e per kg H"[2], ")"))) +
  theme(panel.background = element_rect(fill = "white"), legend.position = "bottom") +
  theme_classic() + 
  theme(text = element_text(family = "Jost", size = 32, color = "black"),
        legend.text = element_text(size = 28),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks = element_line(linewidth = 0.2),
        strip.text = element_text(size = 22, face = "bold"),
        strip.background = element_blank(),
        strip.placement = "bottom") +
  scale_linetype_manual(name = "Emission threshold", 
                        values = c("solid", "solid", "solid", "solid",  "solid"), 
                        labels = expression("US clean H"[2]*"", "EU taxonomy", "UK low carbon H"[2]*"", "Blue H"[2]*"", "Grey H"[2]*""),
                        guide = guide_legend(override.aes = list(color = c("#183919", "#378139", "#74c376", "#6baed6", "#c5c7c8"))))

boxplot_prod_elec_facet

ggsave("Figures/Figure2.tiff", boxplot_prod_elec_facet, width = 18.3, height = 15, units = "cm", dpi = 250)
