################################################################
# Load required packages

library(terra)
library(sp)
library(readxl)

terraOptions(verbose = TRUE, memfrac = 0.3, steps = 10)

################################
# Read in all files

data_dir <- 
  
  # Read in the solar irradiation data, Global Horizontal Irradiation (GHI), in kWh/m2
  # Read data from Global Solar Atlas at https://globalsolaratlas.info
  solar_GHI <- rast(paste0(data_dir,"GlobalSolarAtlas/GHI.tif"))

# Read in panel type data
panel_types <- read.csv("Derived_data/Market_shares_panel_types.csv")

# Read here a shapefile with the world country borders
countries <- vect(paste0(data_dir,"World_country_boundaries/99bfd9e7-bb42-4728-87b5-07f8c8ac631c2020328-1-1vef4ev.lu5nk.shp"))

# Create a raster with countries
countries_raster <- rasterize(countries, solar_data, filename = "countries_raster.tif", overwrite = TRUE)
countries_raster <- rast("countries_raster.tif")

# Read here a shapefile with the exclusive economic zone borders (EEZ)
eez <- vect(paste0(data_dir,"World_EEZ_v11_20191118/eez_v11.shp"))

# Also need the local capacity factor for solar PV a raster.
# Read data for daily solar PV output potential, from https://globalsolaratlas.info.
daily_PVout <- rast(paste0(data_dir,"GlobalSolarAtlas/PVOUT.tif"))

# Read in FTT and link files between countries and EEZ
FTT_elect_countries <- read.csv("FTT_elec_data_countries_3107.csv", sep = ",")
countries_eez_link <- read.csv("EEZ_wind_output_no_overlap.csv", sep = ",")

###############################################

# Calculate the CF for solar for the year 2021, which we use to represent the most recent/ up to date market
solar_GHG <- list()

# Calculate solar_GHG value, based on the location specific solar_GHI
# solar_GHI needs to be multiplied by 365.25 for yearly irradiation in kWh/m2/year
for (i in 1:length(panel_types$panel.type)) {
  if (i == 1) {
    sum_raster <- 0
  } else {
    sum_raster <- solar_GHG[[i-1]]
  }
  
  solar_GHG[[i]] <- sum_raster + panel_types[i,"X2021"] *
    10^(64.3 - 0.03088 * 2021 + panel_types$b_type[i] -
          0.000232 * as.numeric(solar_GHI) * 365.25 + panel_types$b_Y_type[i] * 2021)
}

# Solar_GHG[[5]] is the location specific footprint of solar electricity (gCO2-e/kWh),
# which has summed over the 2021 market shares for different PV technologies multiplied by their respective footprints per kWh
EF_solar <- solar_GHG[[5]]

# Save the new raster with solar_GHG[[5]]
terra::writeRaster(EF_solar, filename = "Solar/solar_GHG_raster.tif", overwrite = TRUE)

####################

# Solar PV output potential per day (kWh/kWp/day) must be divided by 24 to find the solar capacity factor
capacity_factor <- daily_PVout / 24

# Save the capacity factor of solar PV as a raster file. 
terra::writeRaster(capacity_factor, filename = "Solar/capacity_solar.tif", overwrite = TRUE)

#################################################################

elec_CF <- terra::rast("Solar/solar_GHG_raster.tif")

# Create the raster
countries_water <- merge(countries, FTT_elect_countries, by.x = "CNTRY_NAME", by.y = "country_name")
countries_water_raster <- rasterize(countries_water, elec_CF, field = "water_stress", filename = "Solar/countries_raster_water_stress.tif", overwrite = TRUE)
countries_water_raster <- rast("Solar/countries_raster_water_stress.tif")

FTT_elect_countries_eez <- merge(FTT_elect_countries, countries_eez_link, by.x = "NAME", by.y = "country_name", all = TRUE)
eez_water <- merge(eez, FTT_elect_countries, by.x = "MRGID", by.y = "Country_ID")
eez_water_raster <- rasterize(eez_water, elec_CF, field = "water_stress", filename = "Solar/eez_raster_water.tif", overwrite = TRUE)
eez_water_raster <- rast("Solar/eez_raster_water.tif")

water_on_offshore <- terra::lapp(c(countries_water_raster, eez_water_raster), fun = function(x, y) {
  subset1 <- (is.na(x)) & (!is.na(y))
  x[subset1] <- y[subset1]
  return(x)
}, filename = "Solar/countries_eez_raster_water.tif", overwrite = TRUE)
water_on_offshore <- rast("Solar/countries_eez_raster_water.tif")

###################################
# To calculate the footprint of hydrogen production, we need a lot of variables that are independent of the location.
# I define those here, taking PEM electrolysis, to produce 1 kg H2 at 30 bar

# Water.
N_water <- 8.9
N_coolingwater <- 88.1 * 0.2
P_desal <- 3.8 / 1000
E_desal <- 3.0766368
E_deion <- 0.30738884

# Electricity.
electr_req <- 53.7
f_curt <- 0
f_bat <- 0.25
e_bat <- 0.9

# Heat.
heat_req <- 1.0
E_heat_ng <- 42.606407

# Electrolyser infrastructure (balance of plant - BOP, and stacks) given for a 1 MW facility.
# Divide over total amount of hydrogen produced over lifetime, for a given utilisation factor of the electrolyser (0.5)
BOP <- 5.5791144E7
electr_BoP_life <- 20
stacks <- 3.4012868E7
electr_stacks_life <- 7

# Divide over total amount of hydrogen produced over lifetime, for a given utilisation factor of the electrolyser (0.5)
capMW <- 1

# Battery.
cap_bat <- 6
bat_life <- 12
E_bat <- 68.6E6

#################################
# Map 2a: the GHG footprint of hydrogen production (kgCO2-e per kg H2) using solar PV in the Grid-connected: power export configuration (i.e., there is an 'off-taker' of power).

# Read the elec_CF raster
elec_CF <- terra::rast("Solar/solar_GHG_raster.tif")

# Read the capacity factors raster
capacity_factor <- terra::rast("Solar/capacity_solar.tif")

# Resample capacity factor to same resolution
capacity_factor_res <- resample(capacity_factor, elec_CF, filename = "Solar/capacity_solar_resampled.tif")
capacity_factor_res <- terra::rast("Solar/capacity_solar_resampled.tif")

# The utilisation factor of the electrolyser is 0.5, or equal to the local capacity factor if it is larger than 0.5
utilisation_factor <- ifel(capacity_factor_res > 0.5, capacity_factor_res, 0.5)

# This determines how much hydrogen is produced with an electrolyser of a given size (here 1 MW)
ktH2y <- capMW * utilisation_factor * 8760 / (electr_req * 1000)

# Calculate EF_production_offtaker (kgCO2-e per kg H2) based on the fixed parameters 
# and the location-specific elec_CF and utilisaton_factor, and ktH2y
EF_production_offtaker <- 1/1000 * ((N_water + N_coolingwater) * (P_desal * elec_CF * water_on_offshore + E_desal * water_on_offshore + E_deion) + # EF_water
                                      electr_req * elec_CF / (1 - f_curt) * (1 + f_bat / e_bat - f_bat) + # EF_electricity
                                      heat_req * E_heat_ng + # EF_heat
                                      BOP * capMW / (ktH2y * 10^6 * electr_BoP_life) + stacks * capMW / (ktH2y * 10^6 * electr_stacks_life) + # EF_electrolyser
                                      cap_bat * capMW / (ktH2y * 10^6 * electr_BoP_life) * E_bat * (electr_BoP_life / bat_life)) # EF_battery

# Save the new raster as a raster file
terra::writeRaster(EF_production_offtaker, filename = "Solar/EF_production_offtaker_solar.tif", overwrite = TRUE)

# Change resolution for visualization purposes
EF_production_offtaker_km <- terra::aggregate(EF_production_offtaker, fact = 10, fun = "mean", na.rm = TRUE, filename = "Solar/EF_production_offtaker_solar_1km.tif", overwrite = TRUE)
terra::math(EF_production_offtaker_km, fun = "ceiling", filename = "Solar/EF_production_offtaker_solar_1km_int.tif", overwrite = TRUE)

######################################
# Map 2b: the GHG footprint of hydrogen production (kgCO2-e per kg H2) using solar PV in the Off-grid: curtailment configuration.

# Parameter that changes:
f_curt <- 0.5

# Read the elec_CF raster
elec_CF <- terra::rast("solar_GHG_raster.tif")

# Read the capacity factors raster
capacity_factor <- terra::rast("capacity_solar.tif")

# The utilisation factor of the electrolyser is 0.5, or equal to the local capacity factor if it is larger than 0.5
utilisation_factor <- ifel(capacity_factor > 0.5, capacity_factor, 0.5)

# This determines how much hydrogen is produced with an electrolyser of a given size (here 1 MW)
ktH2y <- capMW * utilisation_factor * 8760 / (electr_req * 1000)

# Calculate EF_production_curtailment (kgCO2-e per kg H2) based on the fixed parameters
# and the location-specific elec_CF and utilisaton_factor, and ktH2y
EF_production_curtailment <- 1/1000 * ((N_water + N_coolingwater) * (P_desal * elec_CF * water_on_offshore + E_desal * water_on_offshore + E_deion) + # EF_water
                                         electr_req * elec_CF / (1 - f_curt) * (1 + f_bat / e_bat - f_bat) + # EF_electricity
                                         heat_req * E_heat_ng + # EF_heat
                                         BOP * capMW / (ktH2y * 10^6 * electr_BoP_life) + stacks * capMW / (ktH2y * 10^6 * electr_stacks_life) + # EF_electrolyser
                                         cap_bat * capMW / (ktH2y * 10^6 * electr_BoP_life) * E_bat * (electr_BoP_life / bat_life)) # EF_battery

# Save the new raster as a raster file
terra::writeRaster(EF_production_curtailment, filename = "Solar/EF_production_curtailment_solar.tif", overwrite = TRUE)

# Change resolution for visualization purposes
EF_production_curtailment_km <- terra::aggregate(EF_production_curtailment, fact = 10, fun = "mean", na.rm = TRUE, filename = "Solar/EF_production_curtailment_solar_1km.tif", overwrite = TRUE)
terra::math(EF_production_curtailment_km, fun = "ceiling", filename = "Solar/EF_production_curtailment_solar_1km_int.tif", overwrite = TRUE)

####################################
# Map 2c: the GHG footprint of hydrogen production (kgCO2-e per kg H2) using solar PV in the Grid-connected: power import configuration (hybrid).

# Here more information is needed than only the footprint of wind electricity at each location.
# The electrolyser should run at an electrolyser utilisation factor of 1, so the rest is supplied by a country's grid mix.

# No curtailment
f_curt <- 0
# And no battery in the equation for EF_production_hybrid

# Read the elec_CF raster
elec_CF <- terra::rast("solar_GHG_raster.tif")

# Read the capacity factors raster
capacity_factor <- terra::rast("capacity_solar.tif")

# The utilisation factor of the electrolyser is 1
utilisation_factor <- ifel(capacity_factor >= 0, 1, 1)

# This determines how much hydrogen is produced with an electrolyser of a given size (here 1 MW)
ktH2y <- capMW * utilisation_factor * 8760 / (electr_req * 1000)

# Need to know the footprint of the national electricity mix
countries_elect <- merge(countries, FTT_elect_countries, by.x = "CNTRY_NAME", by.y = "country_name")
countries_elect_raster <- rasterize(countries_elect, elec_CF, field = "GHG_elec_2030_2C", filename = "Solar/countries_raster_FTT.tif", overwrite = TRUE)
countries_elect_raster <- rast("Solar/countries_raster_FTT.tif")

FTT_elect_countries_eez <- merge(FTT_elect_countries, countries_eez_link, by.x = "NAME", by.y = "country_name", all = TRUE)
eez_elect <- merge(eez, FTT_elect_countries, by.x = "MRGID", by.y = "Country_ID")
eez_elect_raster <- rasterize(eez_elect, elec_CF, field = "GHG_elec_2030_2C", filename = "Solar/eez_raster_FTT.tif", overwrite = TRUE)

elect_on_offshore <- terra::lapp(c(countries_elect_raster, eez_elect_raster), fun = function(x, y) {
  subset1 <- (is.na(x)) & (!is.na(y))
  x[subset1] <- y[subset1]
  return(x)
}, filename = "Solar/countries_eez_raster_FTT.tif", overwrite = TRUE)


elect_on_offshore <- rast("Solar/countries_eez_raster_FTT.tif")

elec_CF_hybrid <- (capacity_factor_res * elec_CF) + (1 - capacity_factor_res) * elect_on_offshore

# Calculate EF_production_hybrid (kgCO2-e per kg H2) based on the fixed parameters
# and the location-specific elec_CF_hybrid, utilisaton_factor, ktH2y 
EF_production_hybrid <- 1/1000 * ((N_water + N_coolingwater) * (P_desal * elec_CF_hybrid * water_on_offshore + E_desal * water_on_offshore + E_deion) + # EF_water
                                    electr_req * elec_CF_hybrid / (1 - f_curt) * (1 + f_bat / e_bat - f_bat) + # EF_electricity
                                    heat_req * E_heat_ng + # EF_heat
                                    BOP * capMW / (ktH2y * 10^6 * electr_BoP_life) + stacks * capMW / (ktH2y * 10^6 * electr_stacks_life)) # EF_electrolyser

# Save the new raster as a raster file
terra::writeRaster(EF_production_hybrid, filename = "Solar/EF_production_hybrid_solar.tif", overwrite = TRUE)

# Change resolution for visualization purposes
EF_production_hybrid_km <- terra::aggregate(EF_production_hybrid, fact = 10, fun = "mean", na.rm = TRUE, filename = "Solar/EF_production_hybrid_solar_1km.tif", overwrite = TRUE)
terra::math(EF_production_hybrid_km, fun = "ceiling", filename = "Solar/EF_production_hybrid_solar_1km_int.tif", overwrite = TRUE)
