################################################################
# Load required packages

library(terra)
library(sp)
library(readxl)

terraOptions(verbose = TRUE, memfrac = 0.3, steps = 10)

################################
# Read in all files

data_dir <- 
  
# Read in data for regression to derive windspeeds
input <- read.csv(paste0(data_dir, "Derived_data/WindTurbineGHGData.csv"))
input$log10_EF <- log10(input$GHG.Footprint)
input$v_avg <- input$Average.Wind.Speed
input$on_off <- input$On..Offshore

# Read in the wind speed data, available from https://globalsolaratlas.info.
wind_data <- rast(paste0(data_dir,"GlobalWindAtlas/gwa3_250_wind-speed_100m.tif"))

# Read here a shapefile with the world country borders
countries <- vect(paste0(data_dir,"World_country_boundaries/99bfd9e7-bb42-4728-87b5-07f8c8ac631c2020328-1-1vef4ev.lu5nk.shp"))

# Read here a shapefile with the exclusive economic zone borders (EEZ)
eez <- vect(paste0(data_dir,"World_EEZ_v11_20191118/eez_v11.shp"))

# Read in FTT and link files between countries and EEZ
FTT_elect_countries <- read.csv("FTT_elec_data_countries_3107.csv", sep = ",")
countries_eez_link <- read.csv("EEZ_wind_output_no_overlap.csv", sep = ",")

# Read in the wind capacity factors, available at https://globalsolaratlas.info. Capacity factor class 1 for onshore, class 3 for offshore.
capacity_factor1 <- rast(paste0(data_dir,"GlobalWindAtlas/gwa3_250_capacity-factor_IEC1.tif"))
capacity_factor3 <- rast(paste0(data_dir,"GlobalWindAtlas/gwa3_250_capacity-factor_IEC3.tif"))

#################################################################

# Create a raster with on / off shore

countries_raster <- rasterize(countries, wind_data, field = 1, filename = "countries_raster.tif", overwrite = TRUE)
countries_raster <- rast("countries_raster.tif")

eez_raster <- rasterize(eez, wind_data, field = 0, filename = "eez_raster.tif", overwrite = TRUE)
eez_raster <- rast("eez_raster.tif")

on_off_raster <- eez_raster

on_off_raster_mod <- terra::lapp(c(on_off_raster, countries_raster), fun = function(x, y) {
  x[y == 1] <- 1
  return(x)
}, filename = "on_off_shore.tif", overwrite = TRUE)
on_off_raster_mod <- rast("on_off_shore.tif")


###############################################

# Do the regression
LM.two  <- lm(log10_EF ~ v_avg + on_off, data = input)

# Predict the regression on the rasters
predictors_stack <- c(wind_data, on_off_raster_mod)
names(predictors_stack) <- c("v_avg", "on_off")

# Compute the predicted values
predicted_values <- predict(predictors_stack, LM.two)

# Apply the exponentiation operation
exponentiated_values <- 10 ** predicted_values
exponentiated_values <- 10 ** EF

# Save the exponentiated values as a raster file. 
# These are local the per kWh GHG intensities of wind electricity (gCO2-e / kWh)
terra::writeRaster(exponentiated_values, filename = "wind_GHG_raster.tif", overwrite = TRUE)

#################################################################

# Create a raster with water stress

elec_CF <- terra::rast("wind_GHG_raster.tif")

# Create the raster
countries_water <- merge(countries, FTT_elect_countries, by.x = "CNTRY_NAME", by.y = "country_name")
countries_water_raster <- rasterize(countries_water, elec_CF, field = "water_stress", filename = "countries_raster_water_stress.tif", overwrite = TRUE)
countries_water_raster <- terra::rast("countries_raster_water_stress.tif")

FTT_elect_countries_eez <- merge(FTT_elect_countries, countries_eez_link, by.x = "NAME", by.y = "country_name", all = TRUE)
eez_water <- merge(eez, FTT_elect_countries, by.x = "MRGID", by.y = "Country_ID")
eez_water_raster <- rasterize(eez_water, elec_CF, field = "water_stress", filename = "eez_raster_water.tif", overwrite = TRUE)

water_on_offshore <- terra::lapp(c(countries_water_raster, eez_water_raster), fun = function(x, y) {
  subset1 <- (is.na(x)) & (!is.na(y))
  x[subset1] <- y[subset1]
  return(x)
}, filename = "countries_eez_raster_water.tif", overwrite = TRUE)
water_on_offshore <- rast("countries_eez_raster_water.tif")

#####
# Also need the local capacity factors for onshore and offshore wind as a raster
# The capacity factor is the fraction of the year the wind turbine is producing electricity.

capacity_on_shore <- terra::lapp(c(on_off_raster_mod, capacity_factor1, capacity_factor3), fun = function(x, y, z) {
  subset1 <- (x == 1) & (!is.na(x))
  subset2 <- (x == 0) & (!is.na(x))
  x[subset1] <- y[subset1]
  x[subset2] <- z[subset2]
  return(x)
}, filename = "capacity_on_off_shore.tif", overwrite = TRUE)

###################################
# To calculate the footprint of hydrogen production, we need a lot of variables that are independent of the location.
# I define those here, taking PEM electrolysis, to produce 1 kg H2 at 30 bar

# Water.
N_water <- 8.9
N_coolingwater <- 88.1 * 0.2
P_desal <- 3.8 / 1000
E_desal <- 3.0766368
E_deion <- 0.30738884

# Electricity.
electr_req <- 53.7
f_curt <- 0
f_bat <- 0.25
e_bat <- 0.9

# Heat.
heat_req <- 1.0
E_heat_ng <- 42.606407

# Electrolyser infrastructure (balance of plant - BOP, and stacks) given for a 1 MW facility.
# Divide over total amount of hydrogen produced over lifetime, for a given utilisation factor of the electrolyser (0.5)
BOP <- 5.5791144E7
electr_BoP_life <- 20
stacks <- 3.4012868E7
electr_stacks_life <- 7

# Divide over total amount of hydrogen produced over lifetime, for a given utilisation factor of the electrolyser (0.5)
capMW <- 1

# Battery.
cap_bat <- 6
bat_life <- 12
E_bat <- 68.6E6

#################################
# Map 1a: the GHG footprint of hydrogen production (kgCO2-e per kg H2) using wind power in the Grid-connected: power export configuration (off-taker).

# Read the elec_CF raster
elec_CF <- terra::rast("wind_GHG_raster.tif")

# Read the capacity factors raster
capacity_factor <- terra::rast("capacity_on_off_shore.tif")

# The utilisation factor of the electrolyser is 0.5, or equal to the local capacity factor if it is larger than 0.5
utilisation_factor <- ifel(capacity_factor > 0.5, capacity_factor, 0.5)

# This determines how much hydrogen is produced with an electrolyser of a given size (here 1 MW)
ktH2y <- capMW * utilisation_factor * 8760 / (electr_req * 1000)

# Calculate EF_production_offtaker (kgCO2-e per kg H2) based on the fixed parameters 
# and the location-specific elec_CF and utilisaton_factor, and ktH2y
EF_production_offtaker <- 1/1000 * ((N_water + N_coolingwater) * (P_desal * elec_CF * water_on_offshore + E_desal * water_on_offshore + E_deion) + # EF_water
                                      electr_req * elec_CF / (1 - f_curt) * (1 + f_bat / e_bat - f_bat) + # EF_electricity
                                      heat_req * E_heat_ng + # EF_heat
                                      BOP * capMW / (ktH2y * 10^6 * electr_BoP_life) + stacks * capMW / (ktH2y * 10^6 * electr_stacks_life) + # EF_electrolyser
                                      cap_bat * capMW / (ktH2y * 10^6 * electr_BoP_life) * E_bat * (electr_BoP_life / bat_life)) # EF_battery

# Save the new raster as a raster file
terra::writeRaster(EF_production_offtaker, filename = "EF_production_offtaker_wind.tif", overwrite = TRUE)

# Change resolution for visualization purposes
EF_production_offtaker_km <- terra::aggregate(EF_production_offtaker, fact = 10, fun = "mean", na.rm = TRUE, filename = "EF_production_offtaker_wind_1km.tif", overwrite = TRUE)
terra::math(EF_production_offtaker_km, fun = "ceiling", filename = "EF_production_offtaker_wind_1km_int.tif", overwrite = TRUE)

######################################
# Map 1b: the GHG footprint of hydrogen production (kgCO2-e per kg H2) using wind in the Off-grid: curtailment configuration

# Parameter that changes:
f_curt <- 0.5

# Read the elec_CF raster
elec_CF <- terra::rast("wind_GHG_raster.tif")

# Read the capacity factors raster
capacity_factor <- terra::rast("capacity_on_off_shore.tif")

# The utilisation factor of the electrolyser is 0.5, or equal to the local capacity factor if it is larger than 0.5
utilisation_factor <- ifel(capacity_factor > 0.5, capacity_factor, 0.5)

# This determines how much hydrogen is produced with an electrolyser of a given size (here 1 MW)
ktH2y <- capMW * utilisation_factor * 8760 / (electr_req * 1000)

# Calculate EF_production_curtailment (kgCO2-e per kg H2) based on the fixed parameters
# and the location-specific elec_CF and utilisaton_factor, and ktH2y
EF_production_curtailment <- 1/1000 * ((N_water + N_coolingwater) * (P_desal * elec_CF * water_on_offshore + E_desal * water_on_offshore + E_deion) + # EF_water
                                         electr_req * elec_CF / (1 - f_curt) * (1 + f_bat / e_bat - f_bat) + # EF_electricity
                                         heat_req * E_heat_ng + # EF_heat
                                         BOP * capMW / (ktH2y * 10^6 * electr_BoP_life) + stacks * capMW / (ktH2y * 10^6 * electr_stacks_life) + # EF_electrolyser
                                         cap_bat * capMW / (ktH2y * 10^6 * electr_BoP_life) * E_bat * (electr_BoP_life / bat_life)) # EF_battery

# Save the new raster as a raster file
terra::writeRaster(EF_production_curtailment, filename = "EF_production_curtailment_wind.tif", overwrite = TRUE)

# Change resolution for visualization purposes
EF_production_curtailment_km <- terra::aggregate(EF_production_curtailment, fact = 10, fun = "mean", na.rm = TRUE, filename = "EF_production_curtailment_wind_1km.tif", overwrite = TRUE)
terra::math(EF_production_curtailment_km, fun = "ceiling", filename = "EF_production_curtailment_wind_1km_int.tif", overwrite = TRUE)


####################################
# Map 1c: the GHG footprint of hydrogen production (kgCO2-e per kg H2) using wind in the Grid-connected: power import configuration (hybrid).

# Here more information is needed than only the footprint of wind electricity at each location.
# The electrolyser should run at an electrolyser utilisation factor of 1, so the rest is supplied by a country's grid mix.

# No curtailment
f_curt <- 0
# And no battery in the equation for EF_production_hybrid

# Read the elec_CF raster
elec_CF <- terra::rast("wind_GHG_raster.tif")

# Read the capacity factors raster
capacity_factor <- terra::rast("capacity_on_off_shore.tif")

# The utilisation factor of the electrolyser is 1
utilisation_factor <- ifel(capacity_factor >= 0, 1, 1)

# This determines how much hydrogen is produced with an electrolyser of a given size (here 1 MW)
ktH2y <- capMW * utilisation_factor * 8760 / (electr_req * 1000)

# Need to know the footprint of the national electricity mix
countries_elect <- merge(countries, FTT_elect_countries, by.x = "CNTRY_NAME", by.y = "country_name")
countries_elect_raster <- rasterize(countries_elect, elec_CF, field = "GHG_elec_2030_2C", filename = "countries_raster_FTT.tif", overwrite = TRUE)
countries_elect_raster <- terra::rast("countries_raster_FTT.tif")

FTT_elect_countries_eez <- merge(FTT_elect_countries, countries_eez_link, by.x = "NAME", by.y = "country_name", all = TRUE)
eez_elect <- merge(eez, FTT_elect_countries, by.x = "MRGID", by.y = "Country_ID")
eez_elect_raster <- rasterize(eez_elect, elec_CF, field = "GHG_elec_2030_2C", filename = "eez_raster_FTT2.tif", overwrite = TRUE)
eez_elect_raster <- terra::rast("eez_raster_FTT2.tif")

elect_on_offshore <- terra::lapp(c(countries_elect_raster, eez_elect_raster), fun = function(x, y) {
  subset1 <- (is.na(x)) & (!is.na(y))
  x[subset1] <- y[subset1]
  return(x)
}, filename = "countries_eez_raster_FTT2.tif", overwrite = TRUE)

elect_on_offshore <- rast("countries_eez_raster_FTT2.tif")

elec_CF_hybrid <- (capacity_factor * elec_CF) + (1 - capacity_factor) * elect_on_offshore

# Calculate EF_production_hybrid (kgCO2-e per kg H2) based on the fixed parameters
# and the location-specific elec_CF_hybrid, utilisaton_factor, ktH2y 
EF_production_hybrid <- 1/1000 * ((N_water + N_coolingwater) * (P_desal * elec_CF_hybrid * water_on_offshore + E_desal * water_on_offshore + E_deion) + # EF_water
                                    electr_req * elec_CF_hybrid / (1 - f_curt) * (1 + f_bat / e_bat - f_bat) + # EF_electricity
                                    heat_req * E_heat_ng + # EF_heat
                                    BOP * capMW / (ktH2y * 10^6 * electr_BoP_life) + stacks * capMW / (ktH2y * 10^6 * electr_stacks_life)) # EF_electrolyser

# Save the new raster as a raster file
terra::writeRaster(EF_production_hybrid, filename = "EF_production_hybrid_wind.tif", overwrite = TRUE)

# Change resolution for visualization purposes
EF_production_hybrid_km <- terra::aggregate(EF_production_hybrid, fact = 10, fun = "mean", na.rm = TRUE, filename = "EF_production_hybrid_wind_1km.tif", overwrite = TRUE)
terra::math(EF_production_hybrid_km, fun = "ceiling", filename = "EF_production_hybrid_wind_1km_int.tif", overwrite = TRUE)