library(ggplot2)
library(showtext)
library(gridExtra)

# Set working directory
setwd("")

font_add("Jost", regular = "Scripts/Jost/Jost-400-Book.ttf")
showtext_auto() # turn on showtext auto mode
theme_set(theme_classic(base_family = "Jost")) # set Jost as the base font family

# This script is for creating a figure of production+transport emissions per electricity source over increasing transport distances.

# Set distances in line with what is used to calculate the emissions for each transport mode
distance_pipeline <- c(0, 500, 1000, 2000, 3000, 4000, 5000)
distance_ship <- c(0, 500, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 15000, 20000)

# Read data for grid-connected: power export configuration
dfbase <- read.csv("Derived_data/database_results.csv")

# Only select the Onshore, offshore and solar PV electricity sources.
database_ren <- subset(dfbase, electricity.source == "Onshore wind" | electricity.source == "Offshore wind" | electricity.source == "Solar PV")

# Create one category for 'wind':
database_ren$electricity.source[database_ren$electricity.source == "Onshore wind" | database_ren$electricity.source == "Offshore wind"] <- "Wind"
 
# Reshape data from wide to long format
database_long_pipe <- reshape(database_ren, direction = "long",
                         varying = list(grep("^EF_pipe_lifecycle_", colnames(database_ren))),
                         v.names = "emissions", timevar = "distance_km",
                         times = distance_pipeline, idvar = "id")

# Reshape data from wide to long format
database_long_LH2 <- reshape(database_ren, direction = "long",
                             varying = list(grep("^EF_LH2_lifecycle_", colnames(database_ren))),
                             v.names = "emissions", timevar = "distance_km",
                             times = distance_ship, idvar = "id")

# Reshape data from wide to long format
database_long_NH3S <- reshape(database_ren, direction = "long",
                         varying = list(grep("^EF_lifecycle_H2_NH3_1S_", colnames(database_ren))),
                         v.names = "emissions", timevar = "distance_km",
                         times = distance_ship, idvar = "id")

# Reshape data from wide to long format
database_long_NH3C <- reshape(database_ren, direction = "long",
                              varying = list(grep("^EF_lifecycle_H2_NH3_1C_", colnames(database_ren))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_ship, idvar = "id")


# Convert distance_km to numeric format
database_long_pipe$distance_km <- as.numeric(database_long_pipe$distance_km)
database_long_LH2$distance_km <- as.numeric(database_long_LH2$distance_km)
database_long_NH3S$distance_km <- as.numeric(database_long_NH3S$distance_km)
database_long_NH3C$distance_km <- as.numeric(database_long_NH3C$distance_km)


# Calculate the medians for each electricity source and distance for pipeline data
pipeline_medians <- aggregate(emissions ~ distance_km + electricity.source, data = database_long_pipe, FUN = median)

# Calculate the medians for each electricity source and distance for LH2 data
LH2_medians <- aggregate(emissions ~ distance_km + electricity.source, data = database_long_LH2, FUN = median)


# Calculate the medians for each electricity source and distance for NH3 storage data
NH3S_medians <- aggregate(emissions ~ distance_km + electricity.source, data = database_long_NH3S, FUN = median)

# Calculate the medians for each electricity source and distance for NH3 with cracking data
NH3C_medians <- aggregate(emissions ~ distance_km + electricity.source, data = database_long_NH3C, FUN = median)

# Add transport type column to pipeline_medians
pipeline_medians$type <- "Pipeline"

# Add transport type column to LH2_medians
LH2_medians$type <- "Liquid hydrogen"

# Add transport type column to NH3S_medians
NH3S_medians$type <- "Ammonia with storage"

# Add transport type column to NH3C_medians
NH3C_medians$type <- "Ammonia with cracking"


# Combine the pipeline and LH2 medians
combined_medians <- rbind(pipeline_medians, LH2_medians, NH3C_medians)


database_wind <- subset(dfbase, electricity.source == "Onshore wind" | electricity.source == "Offshore wind")
q25_wind <- quantile(database_wind$EF_production/1000, probs = 0.25)
q75_wind <- quantile(database_wind$EF_production/1000, probs = 0.75)

database_solar <- subset(dfbase, electricity.source == "Solar PV")
q25_solar <- quantile(database_solar$EF_production/1000, probs = 0.25)
q75_solar <- quantile(database_solar$EF_production/1000, probs = 0.75)


# Create the plot
plot_combined_medians <- ggplot() +
  geom_line(data = combined_medians, aes(x = distance_km, y = emissions/1000, color = electricity.source, linetype = type), linewidth = 0.8) +
  geom_rect(aes(ymin = q25_wind, ymax = q75_wind, xmin = -Inf, xmax = Inf, fill = "Wind"), alpha = 0.3) +
  geom_rect(aes(ymin = q25_solar, ymax = q75_solar, xmin = -Inf, xmax = Inf, fill = "Solar PV"), alpha = 0.3) +
  xlab("Distance (km)") +
  ylab(expression(paste("Cradle-to-delivery emissions (kgCO"[~2]*"-e per kg H"[~2]*")"))) +
  scale_x_continuous(expand = c(0, 0), limits = c(0, NA), breaks = seq(0, 20000, by = 5000)) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA), breaks = seq(0, 100, by = 1)) +
  coord_cartesian(ylim = c(0, 12), xlim = c(0, 20000)) +
  scale_color_manual(values = c("Wind" = "#379081", "Solar PV" = "#efd100")) +
  scale_fill_manual(values = c("Wind" = "#379081", "Solar PV" = "#efd100"),
                    breaks = c("Solar PV", "Wind"),
                    labels = c("Solar PV", "Wind")) +
  scale_linetype_manual(values = c("Pipeline" = "solid", "Liquid hydrogen" = "dashed", "Ammonia with cracking" = "dotted"),
                        breaks = c("Pipeline", "Liquid hydrogen", "Ammonia with cracking"),
                        labels = expression("Pipeline", "Liquid H"[2]*" shipping", "Ammonia shipping")) +
  labs(color = "Electricity source", linetype = "Transport type", fill = "Cradle-to-gate emissions") +
  theme_classic() +
  theme(text = element_text(family = "Jost", size = 26, color = "black"),
        legend.text = element_text(size = 26),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks = element_line(linewidth = 0.2))

ggsave("Figures/Figure5A.tiff", plot_combined_medians, width = 18, height = 10, units = "cm", dpi = 300)


#########################
# Only select the solar PV electricity source.
database_solar <- subset(dfbase, electricity.source == "Solar PV")

# Only select the solar PV electricity source.
database_solar_sub <- subset(database_solar, continent == "Europe" | continent == "South America")


# Reshape data from wide to long format
database_long_pipe <- reshape(database_solar_sub, direction = "long",
                              varying = list(grep("^EF_pipe_lifecycle_", colnames(database_solar_sub))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_pipeline, idvar = "id")

# Reshape data from wide to long format
database_long_LH2 <- reshape(database_solar_sub, direction = "long",
                             varying = list(grep("^EF_LH2_lifecycle_", colnames(database_solar_sub))),
                             v.names = "emissions", timevar = "distance_km",
                             times = distance_ship, idvar = "id")

# Reshape data from wide to long format
database_long_NH3S <- reshape(database_solar_sub, direction = "long",
                              varying = list(grep("^EF_lifecycle_H2_NH3_1S_", colnames(database_solar_sub))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_ship, idvar = "id")

# Reshape data from wide to long format
database_long_NH3C <- reshape(database_solar_sub, direction = "long",
                              varying = list(grep("^EF_lifecycle_H2_NH3_1C_", colnames(database_solar_sub))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_ship, idvar = "id")


# Convert distance_km to numeric format
database_long_pipe$distance_km <- as.numeric(database_long_pipe$distance_km)
database_long_LH2$distance_km <- as.numeric(database_long_LH2$distance_km)
database_long_NH3S$distance_km <- as.numeric(database_long_NH3S$distance_km)
database_long_NH3C$distance_km <- as.numeric(database_long_NH3C$distance_km)


# Calculate the medians for each electricity source and distance for pipeline data
pipeline_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_pipe, FUN = median)

# Calculate the medians for each electricity source and distance for LH2 data
LH2_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_LH2, FUN = median)


# Calculate the medians for each electricity source and distance for NH3 storage data
NH3S_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_NH3S, FUN = median)

# Calculate the medians for each electricity source and distance for NH3 with cracking data
NH3C_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_NH3C, FUN = median)

# Add transport type column to pipeline_medians
pipeline_medians$type <- "Pipeline"

# Add transport type column to LH2_medians
LH2_medians$type <- "Liquid hydrogen"

# Add transport type column to NH3S_medians
NH3S_medians$type <- "Ammonia with storage"

# Add transport type column to NH3C_medians
NH3C_medians$type <- "Ammonia with cracking"


# Combine the pipeline and LH2 medians
combined_medians <- rbind(pipeline_medians, LH2_medians, NH3C_medians)


Europe_solar <- database_solar_sub <- subset(database_solar, continent == "Europe")
median(Europe_solar$EF_production)

q25_Europe <- quantile(Europe_solar$EF_production/1000, probs = 0.25)
q75_Europe <- quantile(Europe_solar$EF_production/1000, probs = 0.75)

SA_solar <- database_solar_sub <- subset(database_solar, continent == "South America")
median(SA_solar$EF_production)

q25_SA <- quantile(SA_solar$EF_production/1000, probs = 0.25)
q75_SA <- quantile(SA_solar$EF_production/1000, probs = 0.75)


# Create the plot
plot_Europe_SouthAmerica <- ggplot() +
  geom_line(data = combined_medians, aes(x = distance_km, y = emissions/1000, color = continent, linetype = type), linewidth = 0.8) +
  geom_rect(aes(ymin = q25_Europe, ymax = q75_Europe, xmin = -Inf, xmax = Inf, fill="Europe"), alpha=0.3) +
  geom_rect(aes(ymin = q25_SA, ymax = q75_SA, xmin = -Inf, xmax = Inf, fill = "South America"), alpha = 0.3) +
  xlab("Distance (km)") +
  ylab(expression(paste("Cradle-to-delivery emissions (kgCO"[~2]*"-e per kg H"[~2]*")"))) +
  scale_x_continuous(expand = c(0, 0), limits = c(0, NA), breaks = seq(0, 20000, by = 5000)) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA), breaks = seq(0, 100, by = 1)) +
  coord_cartesian(ylim = c(0, 12), xlim = c(0, 20000)) +  
  scale_color_manual(values = c("Europe" = "#a38e00", "South America" = "#a33d00"),
                     labels = c("Solar PV: Europe", "Solar PV: South America")) +
  scale_fill_manual(values = c("Europe" = "#a38e00", "South America" = "#a33d00"),
                    labels = c("Solar PV: Europe", "Solar PV: South America")) +
  scale_linetype_manual(values = c("Pipeline" = "solid", "Liquid hydrogen" = "dashed", "Ammonia with cracking" = "dotted"),
                        breaks = c("Pipeline", "Liquid hydrogen", "Ammonia with cracking"),
                        labels = expression("Pipeline", "Liquid H"[2]*" shipping", "Ammonia shipping")) +
  labs(color = "Continent", linetype = "Transport type", fill = "Cradle-to-gate emissions") +
  theme_classic() +
  theme(text = element_text(family = "Jost", size = 26, color = "black"),
        legend.text = element_text(size = 26),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks = element_line(linewidth = 0.2))


ggsave("Figures/Fig5B.tiff", plot_Europe_SouthAmerica , width = 18, height = 10, units = "cm", dpi = 300)

#########################
# Only select the Wind onshore electricity source.
database_windon <- subset(dfbase, electricity.source == "Onshore wind")

# Only select the continents Europe and North America
database_windon_sub <- subset(database_windon, continent == "Europe" | continent == "North America")

# Reshape data from wide to long format
database_long_pipe <- reshape(database_windon_sub, direction = "long",
                              varying = list(grep("^EF_pipe_lifecycle_", colnames(database_windon_sub))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_pipeline, idvar = "id")

# Reshape data from wide to long format
database_long_LH2 <- reshape(database_windon_sub, direction = "long",
                             varying = list(grep("^EF_LH2_lifecycle_", colnames(database_windon_sub))),
                             v.names = "emissions", timevar = "distance_km",
                             times = distance_ship, idvar = "id")

# Reshape data from wide to long format
database_long_NH3S <- reshape(database_windon_sub, direction = "long",
                              varying = list(grep("^EF_lifecycle_H2_NH3_1S_", colnames(database_windon_sub))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_ship, idvar = "id")

# Reshape data from wide to long format
database_long_NH3C <- reshape(database_windon_sub, direction = "long",
                              varying = list(grep("^EF_lifecycle_H2_NH3_1C_", colnames(database_windon_sub))),
                              v.names = "emissions", timevar = "distance_km",
                              times = distance_ship, idvar = "id")

# Convert distance_km to numeric format
database_long_pipe$distance_km <- as.numeric(database_long_pipe$distance_km)
database_long_LH2$distance_km <- as.numeric(database_long_LH2$distance_km)
database_long_NH3S$distance_km <- as.numeric(database_long_NH3S$distance_km)
database_long_NH3C$distance_km <- as.numeric(database_long_NH3C$distance_km)

# Calculate the medians for each electricity source and distance for pipeline data
pipeline_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_pipe, FUN = median)

# Calculate the medians for each electricity source and distance for LH2 data
LH2_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_LH2, FUN = median)


# Calculate the medians for each electricity source and distance for NH3 storage data
NH3S_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_NH3S, FUN = median)

# Calculate the medians for each electricity source and distance for NH3 with cracking data
NH3C_medians <- aggregate(emissions ~ distance_km + continent, data = database_long_NH3C, FUN = median)

# Add transport type column to pipeline_medians
pipeline_medians$type <- "Pipeline"

# Add transport type column to LH2_medians
LH2_medians$type <- "Liquid hydrogen"

# Add transport type column to NH3S_medians
NH3S_medians$type <- "Ammonia with storage"

# Add transport type column to NH3C_medians
NH3C_medians$type <- "Ammonia with cracking"


# Combine the pipeline and LH2 medians
combined_medians <- rbind(pipeline_medians, LH2_medians, NH3C_medians)


Europe_windon <- database_windon_sub <- subset(database_windon, continent == "Europe")

q25_Europe <- quantile(Europe_windon$EF_production/1000, probs = 0.25)
q75_Europe <- quantile(Europe_windon$EF_production/1000, probs = 0.75)

NA_windon <- database_windon_sub <- subset(database_windon, continent == "North America")

q25_NA <- quantile(NA_windon$EF_production/1000, probs = 0.25)
q75_NA <- quantile(NA_windon$EF_production/1000, probs = 0.75)


# Create the plot
plot_Europe_NorthAmerica <- ggplot() +
  geom_line(data = combined_medians, aes(x = distance_km, y = emissions/1000, color = continent, linetype = type), linewidth = 0.8) +
  geom_rect(aes(ymin = q25_Europe, ymax = q75_Europe, xmin = -Inf, xmax = Inf, fill="Europe"), alpha=0.3) +
  geom_rect(aes(ymin = q25_NA, ymax = q75_NA, xmin = -Inf, xmax = Inf, fill = "North America"), alpha = 0.3) +
  xlab("Distance (km)") +
  ylab(expression(paste("Cradle-to-delivery emissions (kgCO"[~2]*"-e per kg H"[~2]*")"))) +
  scale_x_continuous(expand = c(0, 0), limits = c(0, NA), breaks = seq(0, 20000, by = 5000)) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA), breaks = seq(0, 100, by = 1)) +
  coord_cartesian(ylim = c(0, 12), xlim = c(0, 20000)) +
  scale_color_manual(values = c("Europe" = "#22594f", "North America" = "#592246"),
                     labels = c("Wind onshore: Europe", "Wind onshore: North America")) +
  scale_fill_manual(values = c("Europe" = "#22594f", "North America" = "#592246"),
                    labels = c("Wind onshore: Europe", "Wind onshore: North America")) +
  scale_linetype_manual(values = c("Pipeline" = "solid", "Liquid hydrogen" = "dashed", "Ammonia with cracking" = "dotted"),
                        breaks = c("Pipeline", "Liquid hydrogen", "Ammonia with cracking"),
                        labels = expression("Pipeline", "Liquid H"[2]*" shipping", "Ammonia shipping")) +
  labs(color = "Continent", linetype = "Transport type", fill = "Cradle-to-gate emissions") +
  theme_classic() +
  theme(text = element_text(family = "Jost", size = 26, color = "black"),
        legend.text = element_text(size = 26),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks = element_line(linewidth = 0.2))


ggsave("Figures/Figure5C.tiff", plot_Europe_NorthAmerica , width = 18, height = 10, units = "cm", dpi = 300)


