library(ggplot2)
library(showtext)
library(gridExtra)

# Set working directory
setwd("")

font_add("Jost", regular = "Scripts/Jost/Jost-400-Book.ttf")
showtext_auto() # turn on showtext auto mode
theme_set(theme_classic(base_family = "Jost")) # set Jost as the base font family

# This script is to create greenhouse gas emission-supply curves for different configurations

# Read emissions data for green hydrogen production for the three configurations
# Grid-connected: power export configuration:
dfbase <- read.csv("Derived_data/database_results.csv")
# Off-grid: curtailment configuration:
dfcurt0.5 <- read.csv("Derived_data/database_results_curt0.5.csv")
# Grid-connected: power import (hybrid) configuration:
dfhybrid <- read.csv("Derived_data/database_results_hybrid1.csv")

# Sort database by footprint in ascending order
sorted_EF_production_dfbase <- dfbase[order(dfbase$EF_production), ]
sorted_EF_production_dfcurt0.5 <- dfcurt0.5[order(dfcurt0.5$EF_production), ]
sorted_EF_production_dfhybrid <- dfhybrid[order(dfhybrid$EF_production), ]


# Create a vector to store the cumulative capacity
cumul_ktH2y <- rep(0, nrow(sorted_EF_production_dfbase))
ktH2ycalc <- rep(0, nrow(sorted_EF_production_dfbase))
cumul_ktH2y_curt0.5 <- rep(0, nrow(sorted_EF_production_dfcurt0.5))
ktH2ycalc_curt0.5 <- rep(0, nrow(sorted_EF_production_dfcurt0.5))
cumul_ktH2y_hybrid <- rep(0, nrow(sorted_EF_production_dfhybrid))
ktH2ycalc_hybrid <- rep(0, nrow(sorted_EF_production_dfhybrid))


# Loop through each row of the database
for (i in 2:nrow(sorted_EF_production_dfbase)) {
  # Get the capacity of the project with lowest footprint
  cumul_ktH2y[1] <- ktH2ycalc[1]
  ktH2ycalc[i] <- sorted_EF_production_dfbase[i, "cap.ktH2_calc"]
  cumul_ktH2y[i] <- ktH2ycalc[i] + cumul_ktH2y[i - 1]
}

for (i in 2:nrow(sorted_EF_production_dfcurt0.5)) {
  cumul_ktH2y_curt0.5[1] <- ktH2ycalc_curt0.5[1]
  ktH2ycalc_curt0.5[i] <- sorted_EF_production_dfcurt0.5[i, "cap.ktH2_calc"]
  cumul_ktH2y_curt0.5[i] <- ktH2ycalc_curt0.5[i] + cumul_ktH2y_curt0.5[i - 1]
}

for (i in 2:nrow(sorted_EF_production_dfhybrid)) {
  cumul_ktH2y_hybrid[1] <- ktH2ycalc_hybrid[1]
  ktH2ycalc_hybrid[i] <- sorted_EF_production_dfhybrid[i, "cap.ktH2_calc"]
  cumul_ktH2y_hybrid[i] <- ktH2ycalc_hybrid[i] + cumul_ktH2y_hybrid[i - 1]
}

# Add to database
sorted_EF_production_dfbase$cumul_ktH2y <- as.numeric(cumul_ktH2y)
sorted_EF_production_dfcurt0.5$cumul_ktH2y <- as.numeric(cumul_ktH2y_curt0.5)
sorted_EF_production_dfhybrid$cumul_ktH2y <- as.numeric(cumul_ktH2y_hybrid)

# Create a new column to indicate the configuration
sorted_EF_production_dfbase$scenario <- "Grid-connected: power export"
sorted_EF_production_dfcurt0.5$scenario <- "Off-grid: curtailment"
sorted_EF_production_dfhybrid$scenario <- "Grid-connected: power import"

# Select the required columns
sorted_EF_production_dfbase <- sorted_EF_production_dfbase[, c("reference", "electricity.source", "EF_production", "cumul_ktH2y", "scenario", "country_name")]
sorted_EF_production_dfcurt0.5 <- sorted_EF_production_dfcurt0.5[, c("reference", "electricity.source", "EF_production", "cumul_ktH2y", "scenario", "country_name")]
sorted_EF_production_dfhybrid <- sorted_EF_production_dfhybrid[, c("reference", "electricity.source", "EF_production", "cumul_ktH2y", "scenario", "country_name")]


# combine the three datasets
combined_df <- rbind(sorted_EF_production_dfbase, sorted_EF_production_dfcurt0.5, sorted_EF_production_dfhybrid)

#create the plot
ESC_prod_MtH2 <- ggplot() + 
  geom_rect(aes(ymin = 10.5, ymax = 17.1, xmin = -Inf, xmax = Inf), fill = "#c5c7c8", alpha = 0.3) +
  geom_rect(aes(ymin = 2.29, ymax = 8.9, xmin = -Inf, xmax = Inf), fill = "#6baed6", alpha = 0.3) +
  geom_hline(aes(yintercept = 4.0, linetype = "US clean"), color = "#183919", linewidth = 0.5) +
  geom_hline(aes(yintercept = 3.0, linetype = "EU taxonomy"), color = "#378139", linewidth = 0.5) +
  geom_hline(aes(yintercept = 2.4, linetype = "UK low carbon"), color = "#74c376", linewidth = 0.5) +
  geom_hline(aes(yintercept = 3.3, linetype = "Blue H2"), color = "#6baed6", linewidth = 0.5) +
  geom_hline(aes(yintercept = 11.5, linetype = "Grey H2"), color = "#c5c7c8", linewidth = 0.5) +
  geom_point(data = subset(combined_df, scenario == "Off-grid: curtailment"), 
             aes(x = cumul_ktH2y / 1000, y = EF_production / 1000, color = "Off-grid: curtailment"), shape = 16, size = 0.5) +
  geom_point(data = subset(combined_df, scenario == "Grid-connected: power import"), 
             aes(x = cumul_ktH2y / 1000, y = EF_production / 1000, color = "Grid-connected: power import"), shape = 16, size = 0.5) +
  geom_point(data = subset(combined_df, !(scenario %in% c("Off-grid: curtailment", "Grid-connected: power import"))), 
             aes(x = cumul_ktH2y / 1000, y = EF_production / 1000, color = electricity.source), shape = 16, size = 1.5) +
  xlab(expression(paste("Cumulative production rate (Mt H"[~2] * " per year)"))) +
  ylab(expression(paste("Cradle-to-gate emissions (kgCO"[2] * "-e per kg H"[2] * ")"))) +
  scale_x_continuous(
    labels = scales::number_format(accuracy = 1),
    breaks = seq(0, max(combined_df$cumul_ktH2y / 1000) + 10, by = 10),
    expand = c(0, 0),
    limits = c(0, max(combined_df$cumul_ktH2y / 1000) + 1)
  ) +
  scale_y_continuous(
    breaks = seq(0, max(combined_df$EF_production / 1000), by = 2),
    expand = c(0, 0),
    limits = c(0, max(combined_df$EF_production / 1000) + 1)
  ) +
  theme_classic() +
  theme(text = element_text(family = "Jost", size = 32, color = "black"),
        legend.text = element_text(size = 28),
        legend.key.size = unit(1, "lines"),
        legend.text.align = 0,
        axis.line = element_line(linewidth = 0.2),
        axis.ticks = element_line(linewidth = 0.2)) +
  scale_color_manual(
    name = "Electricity source",
    values = c(
      "Offshore wind" = "#b3de69",
      "Onshore wind" = "#8dd3c7",
      "Hydropower" = "#80b1d3",
      "Nuclear" = "#fccde5",
      "Renewable mix" = "#d7b2d7",
      "Excess renewable" = "#fdb462",
      "Solar PV" = "#ffed6f",
      "Grid" = "#fc9272",
      "Off-grid: curtailment" = "#969696",
      "Grid-connected: power import" = "#525252"
    ),
    breaks = c(
      "Offshore wind","Onshore wind", "Renewable mix", "Solar PV","Nuclear", "Hydropower", "Excess renewable", "Grid",
      "Off-grid: curtailment",
      "Grid-connected: power import"
    ),
    labels = c(
      "Offshore wind","Onshore wind",  "Renewable mix", "Solar photovoltaics","Nuclear", "Hydropower", "Excess renewable", "Grid",
      "Off-grid: curtailment",
      "Grid-connected: power import"
    )
  ) +
  scale_linetype_manual(name = "Alternative production", 
                        values = c("solid", "solid", "solid", "solid",  "solid"), 
                        labels = expression("US clean H"[2]*"", "EU taxonomy", "UK low carbon H"[2]*"", "Blue H"[2]*"", "Grey H"[2]*""),
                        guide = guide_legend(override.aes = list(color = c("#183919", "#378139", "#74c376", "#6baed6", "#c5c7c8"))))


ESC_prod_MtH2

ggsave("Figures/Figure6A.tiff",ESC_prod_MtH2, width = 18, height = 12, units = "cm", dpi = 300)
