# Set working directory
setwd("")

# This is the script used to quantify the emissions of green hydrogen production and transport in the Off-grid: curtailment configuration.
# In this script and elsewhere, this is also referred to as the 'curtailment' configuration. 
# This script is based on the script for the Grid-connected: power export configuration.
# The main difference is that the surplus of renewable energy resulting from oversizing compared to the electrolyser, is curtailed.
# The curtailment factor is 0.5.

# The main input is the IEA database of hydrogen projects.
# Additional inputs are used to determine the location-specific and renewable energy technology-specific greenhouse gas intensity per kWh of electricity used,
# and to determine the water stress score applicable at the planned location of the hydrogen facility.


# Read data
database <- read.csv("Derived_data/IEA_2022_Hydrogen_db.csv")
country_codes <- read.csv("Derived_data/Country_continent_codes.csv")
country_Knobloch <- read.csv("Derived_data/match_country_Knobloch.csv")
wind_GHGData <- read.csv("Derived_data/WindTurbineGHGData.csv")
elec_grid <- read.csv("Derived_data/Knobloch_grid_2030.csv")
wind_speed <- read.csv("Derived_data/Country_wind_output.csv")
wind_onshore_cf <- read.csv("Derived_data/Country_wind_capacity_output.csv")
wind_offshore_speed <- read.csv("Derived_data/Country_offshore_wind_output.csv")
wind_offshore_cf <- read.csv("Derived_data/Country_offshore_wind_capacity_output.csv")
solar_irradiation <- read.csv("Derived_data/solargis_pv_irradiation.csv")
solar_pvoutput <- read.csv("Derived_data/solargis_pv_output_potential.csv")
panel_types <- read.csv("Derived_data/Market_shares_panel_types.csv")
hydro_class <- read.csv("Derived_data/Hydroelectricity.csv")

# First, we need to select the relevant data from the IEA database for the analysis
# Only include the relevant columns.
database <- database[, c("reference", "name", "country", "date.online", "date.decommissioned", 
                         "technology", "electricity.type", "elec.type.ren", "size.announced.type", 
                         "cap.mwel", "cap.ktH2y", "hydro.elec.class")]

# Select only the electrolysis-based projects
electrolyser_types = c("PEM", "ALK", "SOEC", "Other Electrolysis")
database <- subset(database, technology %in% electrolyser_types)

# For projects with no date.online, we assume the start date is 2030
database$date.online[is.na(database$date.online)] <- 2030
database$date.online[database$date.online == "0"] <- 2030

# Select projects that have a date online of 2030 or before
database <- subset(database, date.online <=2030)

# Exclude projects that are decommissioned in 2030 or before
database <- subset(database, is.na(date.decommissioned) | date.decommissioned > 2030)

# Exclude projects that have no information on the size or capacity
database <- subset(database, size.announced.type == "1" | size.announced.type == "2")

# Exclude projects where the location is missing
database <- subset(database, !country == "")

# For projects where no electricity source is indicated, assign 'Other/Unknown'
database$electricity.type[database$electricity.type == ""] <- "Other/Unknown"

# For Dedicated renewable electricity, merge the options 'Others/Various' and 'Unknown'
database$elec.type.ren[database$elec.type.ren == "Unknown" | database$elec.type.ren == "Others/Various"] <- "Various/Unknown"

# If electricity source is Other/Unknown, assume it's a mix of renewables.
database$electricity.source <- ifelse(database$electricity.type == "Dedicated renewable", database$elec.type.ren,
                                      ifelse(database$electricity.type == "Grid (excess renewable)", "Excess renewable", 
                                             ifelse(database$electricity.type == "Other/Unknown", "Renewable mix", database$electricity.type))) 

# If renewable electricity source is Various/Unknown, assume it's a mix of renewables.
database$electricity.source <- ifelse(database$electricity.source == "Various/Unknown", "Renewable mix", database$electricity.source)

# Unique values in country column
unique_countries <- unique(database$country)
sorted_countries <- sort(unique_countries)

# Add a column that includes the continent for each project
# This simultaneously links the water_stress score, indicating the level of water stress in the country.
database <- merge(database, country_codes, by = "country")

# Add a column that indicates the matching country/region name that Knobloch et al. use for modelling grid electricity GHG emissions.
# This is not always the same at the country name because some regions are modelled in a grouped manner.
database <- merge(database, country_Knobloch, by = "country")


# Here, we define the key parameters for the hydrogen production configuration.
# Specify the electrolyser electricity requirement in kWh per kg H2 for each electrolyser type to produce 1 kg of H2 at 30 bar.
database$elec.req <- as.numeric(ifelse(database$technology == "ALK", 49.7, 
                            ifelse(database$technology == "PEM", 53.7, 
                                   ifelse(database$technology == "SOEC", 42.7,
                                          ifelse(database$technology == "Other Electrolysis", 48.7, NA)))))

# Specify the curtailment factor: here set to 0.5 for intermittent renewables. Treat Other/Unknown electricity as dedicated renewable.
database$f_curtailment <- ifelse(database$electricity.type == "Grid", 0,
                                 ifelse(database$electricity.type == "Nuclear", 0,
                                      ifelse(database$electricity.type == "Grid (excess renewable)", 0,
                                              ifelse(database$electricity.type == "Other/Unknown", 0.5,
                                                    ifelse(database$elec.type.ren == "Hydropower", 0, 0.5)))))


# Specify the energy to power ratio of the battery (cap_bat/cap_electrolyser (h)). Treat Other/Unknown electricity as dedicated renewable.
database$battery_cap_h <- ifelse(database$electricity.type == "Grid", 0,
                                 ifelse(database$electricity.type == "Nuclear", 0,
                                        ifelse(database$electricity.type == "Grid (excess renewable)", 0,
                                               ifelse(database$electricity.type == "Other/Unknown", 6,
                                                      ifelse(database$elec.type.ren == "Hydropower", 0, 6)))))

# Specify the share of electricity passing through the battery. Treat Other/Unknown electricity as dedicated renewable.
database$f_battery <- ifelse(database$electricity.type == "Grid", 0,
                                 ifelse(database$electricity.type == "Nuclear", 0,
                                        ifelse(database$electricity.type == "Grid (excess renewable)", 0,
                                               ifelse(database$electricity.type == "Other/Unknown", 0.25,
                                                      ifelse(database$elec.type.ren == "Hydropower", 0, 0.25)))))


# Quantifying the electricity footprint based on the location and electricity source indicated in the IEA database.
# The electricity footprint is needed to quantify the green hydrogen production emissions. 

# Select the 75th percentile of wind speed (onshore and offshore) (m/s) and solar irradiation (kwh/m2/day),
# and multiply the solar irradiation per day by 365.25 to find the solar radiation per year (kwh/m2/year).
wind_onshore_speed_sub_75 <- data.frame(v_75 = wind_speed$gwa3_250_wind.speed_100m.q75.75.,
                                country_name = wind_speed$country_name)
wind_offshore_speed_sub_75 <- data.frame(v_75 = wind_offshore_speed$gwa3_250_wind.speed_100m.q75.75., 
                                         country_name = wind_offshore_speed$country_name)
solar_irradiation_sub_75 <- data.frame(I_75 = solar_irradiation$X75th_percentile * 365.25,
                                       country = solar_irradiation$country)

# Calculate the wind electricity GHG intensity (gCO2-eq/kWh) per country based on Dammeier et al. (2022).
# The data that Dammeier used to build a GLM with four paramaters is used to create a GLM with only v_avg and onshore/offshore as parameters. 
# The parameter on_off is equal to 0 for onshore turbines and equal to 1 for offshore turbines.
# v_avg is the wind speed at 100 m (m/s) and is location-specific. 

input <- wind_GHGData
input$log10_EF <- log10(input$GHG.Footprint)
input$v_avg <- input$Average.Wind.Speed
input$on_off <- input$On..Offshore
LM.two  <- lm(log10_EF ~ v_avg + on_off, data = input)

wind_onshore_GHG <- data.frame(country_name = wind_onshore_speed_sub_75$country_name, 
                               on_off = as.numeric(0),
                               v_avg = wind_onshore_speed_sub_75$v_75)

wind_offshore_GHG <- data.frame(country_name = wind_offshore_speed_sub_75$country_name, 
                                on_off = as.numeric(1),
                                v_avg = wind_offshore_speed_sub_75$v_75)

# Now apply the GLM to our data and store as wind_onshore_gCO2ekWh and wind_offshore_gCO2ekWh
wind_onshore_GHG <-data.frame(wind_onshore_GHG, wind_onshore_gCO2ekWh = 10**predict(LM.two,newdata=wind_onshore_GHG,type="response"))

wind_offshore_GHG <-data.frame(wind_offshore_GHG, wind_offshore_gCO2ekWh = 10**predict(LM.two,newdata=wind_offshore_GHG,type="response"))


# Calculate the solar electricity GHG intensity (gCO2-eq/kWh) per country based on the metamodel in Bosmans et al. (2021)
# log(EF_solar) = 64.3 - 0.03088*Year + b_type - 0.000232*I + b_Y_type*Year
# b_type and b_Y_type depend on the panel type. We use a weighted average based on the year-specific market mix.

years <- c(2016, 2017, 2018, 2019, 2020, 2021)
colnames(panel_types)[5:10] <- years

solar_GHG <- data.frame(country = solar_irradiation_sub_75$country,
                        I_75 = solar_irradiation_sub_75$I_75,
                        matrix (nrow = length(solar_irradiation_sub_75$country), 
                                ncol = length(years)))
colnames(solar_GHG)[3:8] <- paste0("solar_GHG_", years)

# Initialize solar_GHG[i, j+2] to zero, then let the loop over k add the market shares
# of each panel type for a specific year j.
for (i in 1:length(solar_irradiation_sub_75$country)){
  for (j in 1:length(years)){
    solar_GHG[i, j+2] <- 0
    for(k in 1:length(panel_types$panel.type)){
      solar_GHG[i, j+2] <- solar_GHG[i, j+2] + panel_types[k,j+4]* 
        10^(64.3 - 0.03088*years[j] + panel_types$b_type[k] - 
              0.000232*solar_GHG$I_75[i] + panel_types$b_Y_type[k] * years[j])
    }
  }
}

# Hydroelectricity footprint in gCO2-eq/kWh based on hydro electricity class
hydro_GHG <- data.frame(hydro.elec.class = hydro_class$hydro.elec.class,
                        hydro_gCO2ekWh = hydro_class$mean)

# GHG intensity for nuclear is 27.63 gCO2-eq/kWh (mean)
database$nuclear_gCO2ekWh <- ifelse(database$electricity.type == "Nuclear", 27.63, NA)


# Merge the country-specific wind, hydro and grid GHG intensities with the database.
# 'NA' is returned for those countries for which offshore wind speed or the projects for which hydro is not available/applicable.
database <- merge(database, wind_onshore_GHG, by = "country_name", all.x = TRUE)
database <- merge(database, wind_offshore_GHG, by = "country_name", all.x = TRUE)
database <- merge(database, elec_grid, by = "Knobloch_country", all.x = TRUE)
database <- merge(database, hydro_GHG, by = "hydro.elec.class", all.x = TRUE)

# 'NA' is returned for those countries for which irradiation is not available (Iceland and Finland).
database <- merge(database, solar_GHG, by = "country", all.x = TRUE)

# Create a column with solar PV electricity footprint for the relevant construction year.
database$solar_gCO2ekWh <- ifelse(database$date.online <= 2016, database$solar_GHG_2016,
                           ifelse(database$date.online == 2017, database$solar_GHG_2017,
                           ifelse(database$date.online == 2018, database$solar_GHG_2018,
                           ifelse(database$date.online == 2019, database$solar_GHG_2019,
                           ifelse(database$date.online == 2020, database$solar_GHG_2020,
                                  database$solar_GHG_2021)))))

# Dedicated renewable average/unknown electricity footprint is calculated for each on location, 
# using 50% solar, 25% wind offshore, 25% wind onshore unless one of those sources is not available. 
database$dedicated_ren_gCO2ekWh <- ifelse(is.na(database$solar_gCO2ekWh), 0.5*database$wind_onshore_gCO2ekWh + 0.5* database$wind_offshore_gCO2ekWh,
                                   ifelse(is.na(database$wind_offshore_gCO2ekWh), 0.5*database$solar_gCO2ekWh + 0.5*database$wind_onshore_gCO2ekWh,
                                          0.5*database$solar_gCO2ekWh + 0.25*database$wind_onshore_gCO2ekWh + 0.25*database$wind_offshore_gCO2ekWh)) 

# Determine the country-specific capacity factors of the intermittent renewables.
# Select the 75th percentile of wind capacity factor (onshore and offshore) and solar pv output potential (kWh/kWp/day),
# and divide the pv output potential per day by 24 to find the solar capacity factor.
wind_onshore_cf_sub_75 <- data.frame(wind_on_cf_75 = wind_onshore_cf$gwa3_250_capacity.factor_IEC1.q75.75.,
                                         country_name = wind_onshore_cf$country_name)
wind_offshore_cf_sub_75 <- data.frame(wind_off_cf_75 = wind_offshore_cf$gwa3_250_capacity.factor_IEC3.q75.75., 
                                          country_name = wind_offshore_cf$country_name)
solar_cf_sub_75 <- data.frame(solar_cf_75 = solar_pvoutput$X75th.Percentile / 24,
                                        country = solar_pvoutput$country)

# Merge the country-specific wind and solar capacity factors with the database.
# 'NA' is returned for those countries for which offshore wind speed or irradiation is not available/applicable.
database <- merge(database, wind_onshore_cf_sub_75, by = "country_name")
database <- merge(database, wind_offshore_cf_sub_75, by = "country_name", all.x = TRUE)
database <- merge(database, solar_cf_sub_75, by = "country", all.x = TRUE)

# Determine weighted average of the renewable capacity factors per country. 
# 50% solar, 25% wind offshore, 25% wind onshore unless not available. 
database$ren_cf <- ifelse(is.na(database$solar_cf_75), 0.5*database$wind_on_cf_75 + 0.5* database$wind_off_cf_75,
                   ifelse(is.na(database$wind_off_cf_75), 0.5*database$solar_cf_75 + 0.5*database$wind_on_cf_75,
                          0.5*database$solar_cf_75 + 0.25*database$wind_on_cf_75 + 0.25*database$wind_off_cf_75)) 

# Specify the capacity factors for each electricity source
database$cap.fac <- ifelse(database$electricity.type == "Grid", 1,
                           ifelse(database$electricity.type == "Nuclear", 1,
                                  ifelse(database$electricity.type == "Grid (excess renewable)", 0.05,
                                         ifelse(database$electricity.type == "Other/Unknown", database$ren_cf,
                                                ifelse(database$elec.type.ren == "Hydropower", 1, 
                                                       ifelse(database$elec.type.ren == "Solar PV", database$solar_cf_75,
                                                              ifelse(database$elec.type.ren == "Onshore wind", database$wind_on_cf_75,
                                                                     ifelse(database$elec.type.ren == "Offshore wind",database$wind_off_cf_75,
                                                                            ifelse(database$elec.type.ren == "Various/Unknown", database$ren_cf, 
                                                                                   'NA')))))))))

database$cap.fac <- as.numeric(database$cap.fac)

# Specify the electricity footprint to be used for project, where a footprint of zero is assigned to excess here. 
# For 'Other/Unknown' electricity, use mix of renewables.
database$elec_CF <- ifelse(database$electricity.type == "Grid", database$grid_gCO2ekWh,
                           ifelse(database$electricity.type == "Nuclear", database$nuclear_gCO2ekWh,
                                  ifelse(database$electricity.type == "Grid (excess renewable)", 0,
                                         ifelse(database$electricity.type == "Other/Unknown", database$dedicated_ren_gCO2ekWh,
                                                ifelse(database$elec.type.ren == "Hydropower", database$hydro_gCO2ekWh, 
                                                       ifelse(database$elec.type.ren == "Solar PV", database$solar_gCO2ekWh,
                                                              ifelse(database$elec.type.ren == "Onshore wind", database$wind_onshore_gCO2ekWh,
                                                                     ifelse(database$elec.type.ren == "Offshore wind", database$wind_offshore_gCO2ekWh,
                                                                            ifelse(database$elec.type.ren == "Various/Unknown", database$dedicated_ren_gCO2ekWh, 
                                                                                   'NA')))))))))

database$elec_CF <- as.numeric(database$elec_CF)

# To calculate an average value for renewable electricity, first calculate the average of solar footprints over the years
solar_GHG$avg_years <- (solar_GHG$solar_GHG_2016 + solar_GHG$solar_GHG_2017 + solar_GHG$solar_GHG_2018 +
                          solar_GHG$solar_GHG_2019 + solar_GHG$solar_GHG_2020 + solar_GHG$solar_GHG_2021) / 6

renewable_avg_CF <- 0.25 * mean(wind_offshore_GHG$wind_offshore_gCO2ekWh, na.rm = TRUE) + 
  0.25 * mean(wind_onshore_GHG$wind_onshore_gCO2ekWh, na.rm = TRUE) +
  0.5 * mean(solar_GHG$avg_years, na.rm = TRUE)


# Specify the electrolyser utilisation factor. Treat Other/Unknown electricity as dedicated renewable.
# Using intermittent renewables to their max capacity factor, or assuming a utilisation factor of the electrolyser of 0.5 
database$utilisation.electrolyser <- ifelse(database$electricity.type == "Grid", database$cap.fac,
                                            ifelse(database$electricity.type == "Nuclear", database$cap.fac,
                                                   ifelse(database$electricity.type == "Grid (excess renewable)", database$cap.fac,
                                                          ifelse(database$electricity.type == "Other/Unknown", ifelse(database$cap.fac > 0.5, database$cap.fac, 0.5),
                                                                 ifelse(database$elec.type.ren == "Hydropower", database$cap.fac, 
                                                                        ifelse(database$elec.type.ren == "Solar PV", ifelse(database$cap.fac > 0.5, database$cap.fac, 0.5),
                                                                               ifelse(database$elec.type.ren == "Onshore wind", ifelse(database$cap.fac > 0.5, database$cap.fac, 0.5),
                                                                                      ifelse(database$elec.type.ren == "Offshore wind", ifelse(database$cap.fac > 0.5, database$cap.fac, 0.5), ifelse(database$cap.fac > 0.5, database$cap.fac, 0.5)))))))))


database$utilisation.electrolyser <- as.numeric(database$utilisation.electrolyser)

# Calculate the size of each project in MW if the size was announced as production 'rate' in the IEA database, and make one column of production capacities.
# Here we consider the utilisation factor of the electrolyser based on electricity type and electrolyser-specific requirement.
database$cap.mwel <- as.numeric(database$cap.mwel)

database$cap.MW_calc <- as.numeric(ifelse(database$size.announced.type == 1, database$cap.mwel, 
                                          ifelse(database$size.announced.type == 2, database$cap.ktH2y * 1000 * database$elec.req / (database$utilisation.electrolyser * 8760), NA)))

total_capacity <- sum(database$cap.MW_calc)

# Calculate the production rate of each project in ktH2y if the size was announced as production 'capacity' and make one column of production rates.
database$cap.ktH2_calc <- as.numeric(ifelse(database$size.announced.type == 2, database$cap.ktH2y, 
                                            ifelse(database$size.announced.type == 1, database$cap.mwel * database$utilisation.electrolyser * 8760 / (database$elec.req * 1000), NA)))


# Create a dataframe to store electrolyser-specific operation requirements
electr_data <- data.frame(matrix(ncol=4,nrow=6))
technologies <- unique(database$technology)
colnames(electr_data) <- technologies
rownames(electr_data) <- c("cooling_kgwater", "heat_MJ", "gKOH", "kWh_elec", "BoP_gCO2e", "stacks_gCO2e")

# water requirement for cooling and compressor (kg water / kg H2)
cooling_demand_techs <- c(88.1, 88.1, 644.7, 273.6)

# multiply cooling demand by 0.2 because assuming five recirculation cycles. 
cooling_demand_techs <- cooling_demand_techs * 0.2

# heat demand (MJ / kg H2)
heat_demand_techs <- c(0, 1.0, 18.9, 6.63)

# KOH demand until it needs to be replaced, per MW electrolyser (g KOH / yr / MW)
KOH_demand_techs <- c(65000, 0, 0, 21667)
# to calculate KOH demand per kg H2 later, need to multiply by electrolyser capacity and divide by hydrogen production per year.

# electricity requirement (kWh / kg H2). Incl additional electricity for compression to 30 bar.
electricity_techs <- c(49.7, 53.7, 42.7, 48.7)

# BoP and stacks emissions for each electrolyser technology, for a 1 MW size (gCO2e). 
# To determine infrastructure emissions per kg H2 in a later step, need to divide over total amount of hydrogen produced over the lifetime of the electrolyser.
electr_BoP_techs <- c(8.3361665E7, 5.5791144E7, 1.6013666E8, 9.9763156E7)
electr_stacks_techs <- c(1.8889623E8, 3.4012868E7, 5.8664346E7, 9.3857815E7)

# Assign the values to the appropriate columns
electr_data["cooling_kgwater", "ALK"] <- cooling_demand_techs[1]
electr_data["cooling_kgwater", "PEM"] <- cooling_demand_techs[2]
electr_data["cooling_kgwater", "SOEC"] <- cooling_demand_techs[3]
electr_data["cooling_kgwater", "Other Electrolysis"] <- cooling_demand_techs[4]
electr_data["heat_MJ", "ALK"] <- heat_demand_techs[1]
electr_data["heat_MJ", "PEM"] <- heat_demand_techs[2]
electr_data["heat_MJ", "SOEC"] <- heat_demand_techs[3]
electr_data["heat_MJ", "Other Electrolysis"] <- heat_demand_techs[4]
electr_data["gKOH", "ALK"] <- KOH_demand_techs[1]
electr_data["gKOH", "PEM"] <- KOH_demand_techs[2]
electr_data["gKOH", "SOEC"] <- KOH_demand_techs[3]
electr_data["gKOH", "Other Electrolysis"] <- KOH_demand_techs[4]
electr_data["kWh_elec", "ALK"] <- electricity_techs[1]
electr_data["kWh_elec", "PEM"] <- electricity_techs[2]
electr_data["kWh_elec", "SOEC"] <- electricity_techs[3]
electr_data["kWh_elec", "Other Electrolysis"] <- electricity_techs[4]
electr_data["BoP_gCO2e", "ALK"] <- electr_BoP_techs[1]
electr_data["BoP_gCO2e", "PEM"] <- electr_BoP_techs[2]
electr_data["BoP_gCO2e", "SOEC"] <- electr_BoP_techs[3]
electr_data["BoP_gCO2e", "Other Electrolysis"] <- electr_BoP_techs[4]
electr_data["stacks_gCO2e", "ALK"] <- electr_stacks_techs[1]
electr_data["stacks_gCO2e", "PEM"] <- electr_stacks_techs[2]
electr_data["stacks_gCO2e", "SOEC"] <- electr_stacks_techs[3]
electr_data["stacks_gCO2e", "Other Electrolysis"] <- electr_stacks_techs[4]


# Water parameters.
# 8.9 kg water needed as input per kg of H2. N_water (kg water / kg H2):
N_water <- 8.9

# Water desalination requires 3.8 kWh / m3 water. P_water (kWh / kg H2):
P_desal <- 3.8 / 1000

# Assuming lifetime of desalination plant as in ecoinvent.
# Emissions for desalination excl. electricity, based on adapted ecoinvent process (gCO2e / kg water):
E_desal <- 3.0766368

# Assuming lifetime of deionisation plant as in ecoinvent.
# Emissions for deionisation excl. electricity and water input, based on adapted ecoinvent process (gCO2e / kg water):
E_deion <- 0.30738884

# To calculate emissions of water input EF_water (gCO2e / kg H2), first create a vector to store data
EF_water <- rep(0, nrow(database))

# Loop over the rows of the database
for (i in 1:nrow(database)) {
  # Get the technology and electricity footprint for this row
  tech <- database$technology[i]
  elec_CF <- database$elec_CF[i]
  # get the water stress score for this row, a value between 0 and 5, where 0 is no water stress (so no desalination required) and 5 is high water stress (requiring desalination)
  water_stress <- database$water_stress[i]
  
  # Check which technology is used and calculate the water emissions accordingly
  # multiply the water desalination electricity requirements and the emissions from water desalination with (water_stress/5)
  # a 0 is assigned to landlocked countries as well.
  
  if (tech == "Other Electrolysis") {
    EF_water[i] <- (N_water + electr_data["cooling_kgwater", "Other Electrolysis"]) * (P_desal * elec_CF * (water_stress/5) + E_desal * (water_stress/5) + E_deion)
  } else if (tech == "PEM") {
    EF_water[i] <- (N_water + electr_data["cooling_kgwater", "PEM"]) * (P_desal * elec_CF * (water_stress/5) + E_desal * (water_stress/5) + E_deion)
  } else if (tech == "ALK") {
    EF_water[i] <-(N_water + electr_data["cooling_kgwater", "ALK"]) * (P_desal * elec_CF * (water_stress/5) + E_desal * (water_stress/5) + E_deion)
  } else if (tech == "SOEC") {
    EF_water[i] <- (N_water + electr_data["cooling_kgwater", "SOEC"]) * (P_desal * elec_CF * (water_stress/5) + E_desal * (water_stress/5) + E_deion)
  } else {EF_water[i] <- NA
  }
}


# Add the emissions resulting form the water requirement to the database
database$EF_water <- as.numeric(EF_water)


# Electrolysis. Including electricity, heat, KOH if needed, and infrastructure.

# To create the required heat, use heat from natural gas. Emissions (gCO2e / MJ):
E_heat_ng <- 42.606407
  
# Emissions associated with KOH use (gCO2e / g KOH):
E_KOH <-  2.6666665

# Define lifetime of electrolyser (years)
electr_BoP_life <- 20
electr_stacks_life <- 7

# Round-trip efficiency of the battery, to specify how much of the electricity is lost when passing through the battery.
e_bat <- 0.9

# Define lifetime of battery (10-15 years) so 12 (years).
bat_life <- 12

# Emissions associated with the battery electricity storage capacity (gCO2e / MWh), considering a lifetime of 12 years.
# to find infra emissions per kg H2 in a later step, E_bat must be multiplied by the energy-to-power ratio of the battery cap_bat_h (h), electrolyser capacity (MW), 
# multiplied by (electr_BoP_life / bat_life), and divided by production rate of H2 (kg H2/yr) and lifetime electrolyser (yr).
E_bat <- 68.6E6


# To calculate emissions of electricity in the electrolysis step (gCO2e / kg H2), incl losses from battery use and curtailment, first create a vector to store data
EF_electrolysis_electricity <- rep(0, nrow(database))

# Loop over the rows of the database
for (i in 1:nrow(database)) {
  # Get the electrolyser technology, electricity footprint, electricity curtailment factor and share of electricity passing through the battery for this project and configuration
  tech <- database$technology[i]
  elec_CF <- database$elec_CF[i]
  f_curt <- database$f_curtailment[i]
  f_bat <- database$f_battery[i]
  
  # Check which technology is used and calculate the electrolysis emissions accordingly
  if (tech == "Other Electrolysis") {
    EF_electrolysis_electricity[i] <- electr_data["kWh_elec", "Other Electrolysis"] * elec_CF / (1 - f_curt) * (1 + f_bat / e_bat - f_bat)
  } else if (tech == "PEM") {
    EF_electrolysis_electricity[i] <- electr_data["kWh_elec", "PEM"] * elec_CF / (1 - f_curt)* (1 + f_bat / e_bat - f_bat)
  } else if (tech == "ALK") {
    EF_electrolysis_electricity[i] <- electr_data["kWh_elec", "ALK"] * elec_CF / (1 - f_curt)* (1 + f_bat / e_bat - f_bat)
  } else if (tech == "SOEC") {
    EF_electrolysis_electricity[i] <- electr_data["kWh_elec", "SOEC"] * elec_CF / (1 - f_curt)* (1 + f_bat / e_bat - f_bat)
  } else {EF_electrolysis_electricity[i] <- NA
  }
}

# Add the electrolysis electricity emissions to the database
database$EF_electrolysis_electricity <- as.numeric(EF_electrolysis_electricity)


# To calculate emissions of heat requirement in the electrolysis step (gCO2e / kg H2), first create a vector to store data
EF_electrolysis_heat <- rep(0, nrow(database))

# Loop over the rows of the database
for (i in 1:nrow(database)) {
  # Get the electrolyser technology for this project
  tech <- database$technology[i]
  
  # Check which technology is used and calculate the electrolysis heat emissions accordingly
  if (tech == "Other Electrolysis") {
    EF_electrolysis_heat[i] <- electr_data["heat_MJ", "Other Electrolysis"] * E_heat_ng 
  } else if (tech == "PEM") {
    EF_electrolysis_heat[i] <- electr_data["heat_MJ", "PEM"] * E_heat_ng 
  } else if (tech == "ALK") {
    EF_electrolysis_heat[i] <- electr_data["heat_MJ", "ALK"] * E_heat_ng 
  } else if (tech == "SOEC") {
    EF_electrolysis_heat[i] <- electr_data["heat_MJ", "SOEC"] * E_heat_ng 
  } else {EF_electrolysis_heat[i] <- NA
  }
}

# Add the electrolysis heat emissions to the database
database$EF_electrolysis_heat <- as.numeric(EF_electrolysis_heat)


# To calculate emissions of KOH requirement in electrolysis step (gCO2e / kg H2), first create a vector to store data
EF_electrolysis_KOH <- rep(0, nrow(database))

# Loop over the rows of the database
for (i in 1:nrow(database)) {
  # Get the electrolyser technology,production rate, electrolyser capacity (MW) for this project
  tech <- database$technology[i]
  capMW <- database$cap.MW_calc[i]
  ktH2y <- database$cap.ktH2_calc[i]

  # Check which technology is used and to calculate KOH emissions per kg H2,
  # emissions per g KOH (E_KOH, gCO2e/gKOH) must be multiplied by KOH demand (gKOH/yr/MW), capacity (MW), and divided by kg production per year (ktH2y * 10^6):
  if (tech == "Other Electrolysis") {
    EF_electrolysis_KOH[i] <- electr_data["gKOH", "Other Electrolysis"] * capMW / (ktH2y * 10^6) * E_KOH
  } else if (tech == "PEM") {
    EF_electrolysis_KOH[i] <- electr_data["gKOH", "PEM"] * capMW / (ktH2y * 10^6) * E_KOH 
  } else if (tech == "ALK") {
    EF_electrolysis_KOH[i] <- electr_data["gKOH", "ALK"] * capMW / (ktH2y * 10^6) * E_KOH 
  } else if (tech == "SOEC") {
    EF_electrolysis_KOH[i] <- electr_data["gKOH", "SOEC"] * capMW / (ktH2y * 10^6) * E_KOH 
  } else {EF_electrolysis_KOH[i] <- NA
  }
}

# Add the electrolysis KOH emissions to the database
database$EF_electrolysis_KOH <- as.numeric(EF_electrolysis_KOH)


# To calculate emissions of infrastructure of the electrolyser (gCO2e / kg H2), first create a vector to store data
EF_electrolyser <- rep(0, nrow(database))

# Loop over the rows of the database
for (i in 1:nrow(database)) {
  # Get the electrolyser technology and production rate for this project
  tech <- database$technology[i]
  ktH2y <- database$cap.ktH2_calc[i]
  capMW <- database$cap.MW_calc[i]

  # Check which technology is used and calculate the electrolyser infrastructure emissions accordingly
  # Emissions are given for construction of 1 MW facilities, need to divide by lifetime (yr) of BoP and stack, 
  # and production rate of H2 (kg H2/yr) to get all H2 produced over that time, and multiplied by electrolyser capacity (MW). 
  if (tech == "Other Electrolysis") {
    EF_electrolyser[i] <- electr_data["BoP_gCO2e", "Other Electrolysis"] * capMW / (ktH2y * 10^6 * electr_BoP_life) +
      electr_data["stacks_gCO2e", "Other Electrolysis"] * capMW / (ktH2y * 10^6 * electr_stacks_life)
  } else if (tech == "PEM") {
    EF_electrolyser[i] <- electr_data["BoP_gCO2e", "PEM"] * capMW / (ktH2y * 10^6 * electr_BoP_life) +
      electr_data["stacks_gCO2e", "PEM"] * capMW / (ktH2y * 10^6 * electr_stacks_life)
  } else if (tech == "ALK") {
    EF_electrolyser[i] <- electr_data["BoP_gCO2e", "ALK"] * capMW / (ktH2y * 10^6 * electr_BoP_life) +
      electr_data["stacks_gCO2e", "ALK"] * capMW / (ktH2y * 10^6 * electr_stacks_life)
  } else if (tech == "SOEC") {
    EF_electrolyser[i] <- electr_data["BoP_gCO2e", "SOEC"] * capMW / (ktH2y * 10^6 * electr_BoP_life) +
      electr_data["stacks_gCO2e", "SOEC"] * capMW / (ktH2y * 10^6 * electr_stacks_life)
  } else {EF_electrolyser[i] <- NA
  }
}

# Add the electrolyser infrastructure emissions to the database
database$EF_electrolyser <- as.numeric(EF_electrolyser)


# To calculate emissions of the battery manufacturing (gCO2e / kg H2), first create a vector to store data
EF_battery <- rep(0, nrow(database))

# Loop over the rows of the database
for (i in 1:nrow(database)) {
  # Get the production rate, electrolyser capacity, and relative battery capacity for this project
  ktH2y <- database$cap.ktH2_calc[i]
  capMW <- database$cap.MW_calc[i]
  cap_bat <- database$battery_cap_h[i]
  
  EF_battery[i] <- cap_bat * capMW / (ktH2y * 10^6 * electr_BoP_life) * E_bat * (electr_BoP_life / bat_life)
}

# Add the battery emissions to the database
database$EF_battery <- as.numeric(EF_battery)

# Total emissions (kgCO2e / kg H2) of the production phase
database$EF_production <- database$EF_water + database$EF_electrolysis_electricity + database$EF_electrolysis_heat + database$EF_electrolysis_KOH + database$EF_electrolyser + database$EF_battery


# Next up is Transport.

# GWP H2 (gCO2e / kg H2) is 11E3
GWP_H2 <- 11E3


# Pipeline.

# Compression from 30 bar to 80 bar electricity requirement (kWh / kg H2)
P_pipeline_comp <- 0.71

# Infrastructure emissions of the compressor per 32 MW compressor (gCO2e)
E_32MW_comp <- 8.7558766E8

# Lifetime compressor (years)
comp_life <- 20

# For continuous production (24h per day, 365 h/year), the compressor's infra emissions (gCO2e / kWh)
E_comp_kWh <- E_32MW_comp * 10^-3 / (32 * 24 * 365 * comp_life)
# This must be multiplied by kWh required in the compression step, and
# needs to be divided by the utilisation factor to account for less use in its life.

# Compressor loss of H2 (kg H2/ kg H2)
L_comp <- 0.5E-2

# Re-compression electricity (kWh / km)
P_pipeline_recomp <- 0.426E-3

# Pipeline losses due to re-compression, assuming a compressor each 125 km (kg H2/ km)
L_pipeline <- 0.004E-2

# Pipeline and compression manufacturing emissions (gCO2e / kg H2 / km)
E_pipeline <- 0.24

# Re-compression at destination location (kWh / kg H2)
P_pipeline_destination_recomp <- 0.24

# We determine the emissions associated with the storage vessel of gaseous H2 at the destination.
# Emissions associated with the storage vessel of 527 kg H2 infra (gCO2e)
E_H2g_storage <- 6.6583583E8 
vessel_kg_H2 <- 527

# Storage days at destination (days)
storage_H2g_dest_day <- 3

# Lifetime of storage vessel (years)
storage_H2g_life <- 20

# Emissions associated with storage infra at destination per kg H2 (gCO2e / kg H2)
H2g_dest_storage_gCO2e <- E_H2g_storage * storage_H2g_dest_day / 365 / vessel_kg_H2 / storage_H2g_life

# Set distances
distance_pipeline <- c(0, 500, 1000, 2000, 3000, 4000, 5000)

# Create vectors to store the emissions for steps in pipeline transport independent of distance (gCO2e / kg H2)
# 1st compression step, re-compression at destination, and gaseous hydrogen storage at destination.
EF_pipe_comp1 <- rep(0, nrow(database))
EF_pipe_comp2 <- rep(0, nrow(database))
EF_pipe_storage <- rep(0, nrow(database))

# Loop through each row of the database
for (i in 1:nrow(database)) {
  # Get the local electricity footprint
  # use average renewable electricity footprint renewable_avg_CF for re-compression
  elec_CF <- database$elec_CF[i]
  f_utilisation <- database$utilisation.electrolyser[i]

  # Calculate the emissions factor hydrogen compression initially (comp1) and at the destination (comp2), 
  # including the loss, s.t. 1 kg of hydrogen is left at the end of this step. 
  EF_pipe_comp1[i] <- (1 / (1 - L_comp)) * (P_pipeline_comp * (elec_CF + E_comp_kWh / f_utilisation) + L_comp * GWP_H2)
  EF_pipe_comp2[i] <- (1 / (1 - L_comp)) * (P_pipeline_destination_recomp * (renewable_avg_CF  + E_comp_kWh / f_utilisation) + L_comp * GWP_H2)
  EF_pipe_storage[i] <- H2g_dest_storage_gCO2e
}

# Add the emissions to the database
database$EF_pipe_comp1 <- as.numeric(EF_pipe_comp1)
database$EF_pipe_comp2 <- as.numeric(EF_pipe_comp2)
database$EF_pipe_storage <- as.numeric(EF_pipe_storage)


# Create a vector to store the pipeline emissions factors for each distance (gCO2e / kg H2)
EF_pipeline <- list()

# Create vectors to store the additional emissions due to losses further upstream for each stage, for each distance (gCO2e / kg H2)
# Only to the step before the last loss (i.e., before the lost compression step in this case)
EF_pipe_H2prod_add <- list()
EF_pipe_comp1_add <- list()
EF_pipeline_add <- list()

# Create a vector to store the full pipeline emissions factors for each distance (gCO2e / kg H2)
EF_pipe_lifecycle <- list()

# Create vectors to store total emissions for each step (production + to make up for losses (additional emissions)) (gCO2e / kg H2)
EF_pipe_H2prod_tot <- list()
EF_pipe_comp1_tot <- list()
EF_pipeline_tot <- list()

# Create a vector to store total H2 loss during value chain in case of pipeline transport (gCO2e / kg H2)
EF_pipe_additions <- list()

# Create a vector to store transport part (excl production) in case of pipeline transport, per kg H2 (without making up for losses) and total (making up for losses) (gCO2e / kg H2)
EF_pipeline_transport <- list()
EF_pipeline_transport_tot <- list()


# Loop through each distance
for (d in 1:length(distance_pipeline)) {
  
  # Create a vector to store data for this distance
  EF_pipeline[[d]] <- rep(0, nrow(database))
  EF_pipe_H2prod_add[[d]] <- rep(0, nrow(database))
  EF_pipe_comp1_add[[d]] <- rep(0, nrow(database))
  EF_pipeline_add[[d]] <- rep(0, nrow(database))
  EF_pipe_lifecycle[[d]] <- rep(0, nrow(database))
  EF_pipe_H2prod_tot[[d]] <- rep(0, nrow(database))
  EF_pipe_comp1_tot[[d]] <- rep(0, nrow(database))
  EF_pipeline_tot[[d]] <- rep(0, nrow(database))  
  EF_pipe_additions[[d]] <- rep(0, nrow(database))
  EF_pipeline_transport[[d]] <- rep(0, nrow(database))
  EF_pipeline_transport_tot[[d]] <- rep(0, nrow(database))
  
  # Loop through each row of the database
  for (i in 1:nrow(database)) {
    # Get the local electricity footprint and H2 production footprint for this project
    elec_CF <- database$elec_CF[i]
    prod_H2_EF <- database$EF_production[i]
    comp1_EF <- database$EF_pipe_comp1[i]
    comp2_EF <- database$EF_pipe_comp2[i]
    storage_EF <- database$EF_pipe_storage[i]
    
    # Calculate the emissions for pipeline transport for a specific distance, incl. loss, s.t. 1 kg of hydrogen at end of this step
    EF_pipeline[[d]][i] <- (1 / (1 - L_pipeline * distance_pipeline[d])) * E_pipeline * distance_pipeline[d] + L_pipeline * distance_pipeline[d] * GWP_H2
    
    
    EF_pipe_H2prod_add[[d]][i] <-  ((1 / (1 - L_comp)) * (1 / (1 - L_pipeline * distance_pipeline[d])) * (1 / (1 - L_comp)) -1) * prod_H2_EF
      
    EF_pipe_comp1_add[[d]][i] <- ((1 / (1 - L_pipeline * distance_pipeline[d])) * (1 / (1 - L_comp)) -1 ) * comp1_EF
      
    EF_pipeline_add[[d]][i] <- ((1 / (1 - L_comp)) - 1) * EF_pipeline[[d]][i]
    
    EF_pipe_lifecycle[[d]][i] <- prod_H2_EF + EF_pipe_H2prod_add[[d]][i] + comp1_EF + EF_pipe_comp1_add[[d]][i] +
                                    EF_pipeline[[d]][i] + EF_pipeline_add[[d]][i] + comp2_EF + storage_EF
    
    # Calculate the emissions for each step including the added emissions from upstream
    EF_pipe_H2prod_tot[[d]][i] <- prod_H2_EF + EF_pipe_H2prod_add[[d]][i]
    
    EF_pipe_comp1_tot[[d]][i] <- comp1_EF + EF_pipe_comp1_add[[d]][i]
    
    EF_pipeline_tot[[d]][i] <- EF_pipeline[[d]][i] + EF_pipeline_add[[d]][i]
    
    # Total emissions to make up for upstream losses
    EF_pipe_additions[[d]][i] <- EF_pipe_H2prod_add[[d]][i] + EF_pipe_comp1_add[[d]][i] + EF_pipeline_add[[d]][i]
    
    # Transport part (excl production), per kg H2 (without making up for losses) and total (making up for losses) (gCO2e / kg H2)
    EF_pipeline_transport[[d]][i] <- comp1_EF + EF_pipeline[[d]][i] + comp2_EF + storage_EF
    EF_pipeline_transport_tot[[d]][i] <- comp1_EF + EF_pipe_comp1_add[[d]][i] +
      EF_pipeline[[d]][i] + EF_pipeline_add[[d]][i] + comp2_EF + storage_EF
    
  }
  
  # Add the pipeline transport emissions to the database for this distance
  database[[paste0("EF_pipeline_", distance_pipeline[d])]] <- as.numeric(EF_pipeline[[d]])
  database[[paste0("EF_pipe_H2prod_add_", distance_pipeline[d])]] <- as.numeric(EF_pipe_H2prod_add[[d]])
  database[[paste0("EF_pipe_comp1_add_", distance_pipeline[d])]] <- as.numeric(EF_pipe_comp1_add[[d]])
  database[[paste0("EF_pipeline_add_", distance_pipeline[d])]] <- as.numeric(EF_pipeline_add[[d]])
  database[[paste0("EF_pipe_lifecycle_", distance_pipeline[d])]] <- as.numeric(EF_pipe_lifecycle[[d]])
  database[[paste0("EF_pipe_H2prod_tot_", distance_pipeline[d])]] <- as.numeric(EF_pipe_H2prod_tot[[d]])
  database[[paste0("EF_pipe_comp1_tot_", distance_pipeline[d])]] <- as.numeric(EF_pipe_comp1_tot[[d]])
  database[[paste0("EF_pipeline_tot_", distance_pipeline[d])]] <- as.numeric(EF_pipeline_tot[[d]])
  database[[paste0("EF_pipeline_transport_", distance_pipeline[d])]] <- as.numeric(EF_pipeline_transport[[d]])
  database[[paste0("EF_pipeline_transport_tot_", distance_pipeline[d])]] <- as.numeric(EF_pipeline_transport_tot[[d]])
  
  # Add loss to the database for this distance
  database[[paste0("EF_pipe_additions_", distance_pipeline[d])]] <- as.numeric(EF_pipe_additions[[d]])
}



# Shipping via liquid hydrogen.

# Electricity requirement for liquefaction (kWh / kg H2)
P_liqf <- 12

# Liquefaction plant (capacity 50 tH2/day continuous operation, incl 21 days storage) emissions (gCO2).
E_liqf <- 1.47957E10
liqf_plant_kgday <- 50E3

# Lifetime liquefaction plant (y)
liqf_life <- 30

# To calculate emissions (gCO2e / kg H2) for liqf plant need to divide by daily production * operational days in lifetime:
# E_liqf / (liqf_plant_kgday * utilisation.electrolyser * 365 * lifq_life)

# Loss during liquefaction (kg H2/ kg H2)
L_liqf <- 1.625E-2

# Tanker shipping mass (kt H2)
m_tanker_H2 <- 11.376

# Liquid storage duration before shipping, max is 21 days (days)
database$liq_storage_days <- ifelse(m_tanker_H2 / (database$cap.ktH2_calc / 365) > 21, 21,
                                    m_tanker_H2 / (database$cap.ktH2_calc / 365))

# Leakage during storage as liquid H2 (kg H2 / kg H2)
L_liq_storage <- 0.16E-2

# BOG per day during storage (kg H2/ kg H2/ day)
BOG_LH2_storage <- 0.18E-2

# BOG is re-compressed to make it ready for liquefaction, requires electricity (kWh / kg BOG H2)
P_BOG_comp <- 1.81

# Liquid H2 loading electricity requirement (kWh / kg H2)
P_LH2_loading <- 0.198

# BOG during loading (loss) (kg H2/ kg H2)
BOG_LH2_loading <- 0.52E-2

# Shipping days depend on the distance travelled. 
# Shipping speed (16 knots) in km/day
v_shipping <- 16 * 1.852 * 24

# Shipping (ballast and laden voyage) (gCO2e per kg*km)
E_LH2_ship <- (45.841018 + 43.577135) * 10^-3

# BOG during shipping (kg H2/ kg H2/ day)
BOG_LH2_shipping <- 0.3E-2

# Liquid H2 unloading electricity requirement (kWh / kg H2)
P_LH2_unloading <- 0.182

# BOG during unloading (loss) (kg H2 / kg H2)
BOG_LH2_unloading <- 0.52E-2

# Regasification electricity (kWh / kg H2)
P_LH2_regasification <- 0

# Storage of gaseous H2 at the destination is the same as for pipeline transport. 

# Create a vector to store the LH2 production emissions factors (gCO2e / kg H2)
EF_LH2_liqf <- rep(0, nrow(database))
EF_LH2_storage <- rep(0, nrow(database))
EF_LH2_loading <- rep(0, nrow(database))
EF_LH2_unloading <- rep(0, nrow(database))
EF_LH2_regas_comp <- rep(0, nrow(database))
EF_LH2_H2g_storage <- rep(0, nrow(database))

# Loop through each row of the database
for (i in 1:nrow(database)) {
  # Get the local electricity footprint and H2 prod emissions, storage days LH2
  elec_CF <- database$elec_CF[i]
  prod_H2_EF <- database$EF_production[i]
  storage_days <- database$liq_storage_days[i]
  f_utilisation <- database$utilisation.electrolyser[i]
 
  # Calculate the emissions factor for this row. Emissions for infra of liquefaction + storage are included here.
  EF_LH2_liqf[i] <- (1 / (1 - L_liqf)) * ((P_liqf * elec_CF + E_liqf / (liqf_plant_kgday * f_utilisation * 365 * liqf_life)) + L_liqf * GWP_H2)
  
  # Adding loss and the part of BOG that is lost in compression and reliquefaction to produce 1 kg output
  EF_LH2_storage[i] <- (1 / (1 - L_liq_storage)) * (1 / (1 - BOG_LH2_storage * storage_days * L_comp)) * (1 / (1 - BOG_LH2_storage * storage_days * (1 - L_comp) * L_liqf)) * 
    ((BOG_LH2_storage * storage_days * P_BOG_comp * (elec_CF + E_comp_kWh / f_utilisation) + BOG_LH2_storage * storage_days * (1 - L_comp) * P_liqf * elec_CF) + 
    (L_liq_storage + BOG_LH2_storage * storage_days * L_comp + BOG_LH2_storage * storage_days * (1 - L_comp) * L_liqf) * GWP_H2) 

  EF_LH2_loading[i] <- (1 / (1 - BOG_LH2_loading)) * (P_LH2_loading * elec_CF + BOG_LH2_loading * GWP_H2)
    
  EF_LH2_unloading[i] <- (1 / (1 - BOG_LH2_unloading)) * (P_LH2_unloading * renewable_avg_CF + BOG_LH2_unloading * GWP_H2)
    
  EF_LH2_regas_comp[i] <- (1 / (1 - L_comp)) * (P_LH2_regasification * (renewable_avg_CF + E_comp_kWh / f_utilisation) + L_comp * GWP_H2)
  
  EF_LH2_H2g_storage[i] <- H2g_dest_storage_gCO2e
  
  }

# Add the LH2 production emissions to the database
database$EF_LH2_liqf <- as.numeric(EF_LH2_liqf)
database$EF_LH2_storage <- as.numeric(EF_LH2_storage)
database$EF_LH2_loading <- as.numeric(EF_LH2_loading)
database$EF_LH2_unloading <- as.numeric(EF_LH2_unloading)
database$EF_LH2_regas_comp <- as.numeric(EF_LH2_regas_comp)
database$EF_LH2_H2g_storage <- as.numeric(EF_LH2_H2g_storage)


# Set distances
distance_ship <- c(0, 500, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000,
                   15000, 20000)


# Create vectors to store the LH2 shipping emissions factors for each distance (gCO2e / kg H2)
EF_LH2_ship <- list()

# Create vectors to store the additional emissions due to losses further upstream for each stage, for each distance (gCO2e / kg H2)
# Only to the step before the last loss (i.e., before the last compression step in this case)
EF_LH2_H2prod_add <- list()
EF_LH2_liqf_add <- list()
EF_LH2_storage_add <- list()
EF_LH2_loading_add <- list()
EF_LH2_ship_add <- list()
EF_LH2_unloading_add <- list()

# Create a vector to store the full LH2 emissions factors for each distance (gCO2e / kg H2)
EF_LH2_lifecycle <- list()
EF_LH2_additions <- list()

# Create vectors to store the total emissions per step, incl additional emissions due to losses further upstream for each stage, for each distance (gCO2e / kg H2)
EF_LH2_H2prod_tot <- list()
EF_LH2_liqf_tot <- list()
EF_LH2_storage_tot <- list()
EF_LH2_loading_tot <- list()
EF_LH2_ship_tot <- list()
EF_LH2_unloading_tot <- list()

# Total emissions for everything after production
EF_LH2_transport <- list()
EF_LH2_transport_tot <- list()


# Loop through each distance
for (d in 1:length(distance_ship)) {
  
  # Create vectors to store data for this distance
  EF_LH2_ship[[d]] <- rep(0, nrow(database))
  EF_LH2_H2prod_add[[d]] <- rep(0, nrow(database))
  EF_LH2_liqf_add[[d]] <- rep(0, nrow(database))
  EF_LH2_storage_add[[d]] <- rep(0, nrow(database))
  EF_LH2_loading_add[[d]] <- rep(0, nrow(database))
  EF_LH2_ship_add[[d]] <- rep(0, nrow(database))
  EF_LH2_unloading_add[[d]] <- rep(0, nrow(database))
  EF_LH2_lifecycle[[d]] <- rep(0, nrow(database))
  EF_LH2_additions[[d]] <- rep(0, nrow(database))
  EF_LH2_H2prod_tot[[d]] <- rep(0, nrow(database))
  EF_LH2_liqf_tot[[d]] <- rep(0, nrow(database))
  EF_LH2_storage_tot[[d]] <- rep(0, nrow(database))
  EF_LH2_loading_tot[[d]] <- rep(0, nrow(database))
  EF_LH2_ship_tot[[d]] <- rep(0, nrow(database))
  EF_LH2_unloading_tot[[d]] <- rep(0, nrow(database))
  EF_LH2_transport[[d]] <- rep(0, nrow(database))
  EF_LH2_transport_tot[[d]] <- rep(0, nrow(database))
  
  # Loop through each row of the database
  for (i in 1:nrow(database)) {
    # Get the local electricity footprint and H2 production footprint for this project
    elec_CF <- database$elec_CF[i]
    prod_H2_EF <- database$EF_production[i]
    liqf_EF <- database$EF_LH2_liqf[i]
    LH2_storage_EF <- database$EF_LH2_storage[i]
    loading_EF <- database$EF_LH2_loading[i]
    unloading_EF <- database$EF_LH2_unloading[i]
    regas_comp_EF <- database$EF_LH2_regas_comp[i]
    storage_H2g_EF <- database$EF_LH2_H2g_storage[i]
    
    # Calculate the emissions factor for this row and distance, incl. loss, s.t. 1 kg of hydrogen at end of this step
    EF_LH2_ship[[d]][i] <- (1 / (1 - BOG_LH2_shipping * distance_ship[d] / v_shipping)) * E_LH2_ship * distance_ship[d] + 
      (BOG_LH2_shipping * distance_ship[d] / v_shipping) * GWP_H2

    EF_LH2_H2prod_add[[d]][i] <- ((1 / (1 - L_liqf)) * (1 / (1 - L_liq_storage)) * (1 / (1 - BOG_LH2_storage * storage_days * L_comp)) * (1 / (1 - BOG_LH2_storage * storage_days * (1 - L_comp) * L_liqf)) *
                                (1 / (1 - BOG_LH2_loading)) * (1 / (1 - BOG_LH2_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_LH2_unloading)) * (1 / (1 - L_comp)) - 1) * prod_H2_EF
      
    EF_LH2_liqf_add[[d]][i] <- ((1 / (1 - L_liq_storage)) * (1 / (1 - BOG_LH2_storage * storage_days * L_comp)) * (1 / (1 - BOG_LH2_storage * storage_days * (1 - L_comp) * L_liqf)) *
                                  (1 / (1 - BOG_LH2_loading)) * (1 / (1 - BOG_LH2_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_LH2_unloading)) * (1 / (1 - L_comp)) - 1) * liqf_EF
      
    EF_LH2_storage_add[[d]][i] <- ((1 / (1 - BOG_LH2_loading)) * (1 / (1 - BOG_LH2_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_LH2_unloading)) * (1 / (1 - L_comp)) - 1) * LH2_storage_EF
      
    EF_LH2_loading_add[[d]][i] <- ((1 / (1 - BOG_LH2_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_LH2_unloading)) * (1 / (1 - L_comp)) - 1) * loading_EF
      
    EF_LH2_ship_add[[d]][i] <- ((1 / (1 - BOG_LH2_unloading)) * (1 / (1 - L_comp)) - 1) * EF_LH2_ship[[d]][i]
    
    EF_LH2_unloading_add[[d]][i] <- ((1 / (1 - L_comp)) - 1) * unloading_EF
      
    EF_LH2_lifecycle[[d]][i] <- prod_H2_EF + EF_LH2_H2prod_add[[d]][i] + liqf_EF + EF_LH2_liqf_add[[d]][i] + 
      LH2_storage_EF + EF_LH2_storage_add[[d]][i] + loading_EF + EF_LH2_loading_add[[d]][i] +
      EF_LH2_ship[[d]][i] + EF_LH2_ship_add[[d]][i] + unloading_EF + EF_LH2_unloading_add[[d]][i]+
      regas_comp_EF + storage_H2g_EF
    
    EF_LH2_additions[[d]][i] <- EF_LH2_H2prod_add[[d]][i] + EF_LH2_liqf_add[[d]][i] + 
      EF_LH2_storage_add[[d]][i] + EF_LH2_loading_add[[d]][i] +
      EF_LH2_ship_add[[d]][i] + EF_LH2_unloading_add[[d]][i]
    
    
    EF_LH2_H2prod_tot[[d]][i] <- prod_H2_EF + EF_LH2_H2prod_add[[d]][i]
    
    EF_LH2_liqf_tot[[d]][i] <- liqf_EF + EF_LH2_liqf_add[[d]][i]
    
    EF_LH2_storage_tot[[d]][i] <- LH2_storage_EF + EF_LH2_storage_add[[d]][i]
    
    EF_LH2_loading_tot[[d]][i] <- loading_EF + EF_LH2_loading_add[[d]][i]
    
    EF_LH2_ship_tot[[d]][i] <- EF_LH2_ship[[d]][i] +  EF_LH2_ship_add[[d]][i]
    
    EF_LH2_unloading_tot[[d]][i] <- unloading_EF + EF_LH2_unloading_add[[d]][i]
    
    EF_LH2_transport[[d]][i] <- liqf_EF + LH2_storage_EF + loading_EF + EF_LH2_ship[[d]][i] + 
      unloading_EF + regas_comp_EF + storage_H2g_EF
    
    EF_LH2_transport_tot[[d]][i] <- liqf_EF + EF_LH2_liqf_add[[d]][i] + LH2_storage_EF + EF_LH2_storage_add[[d]][i] + 
      loading_EF + EF_LH2_loading_add[[d]][i] + EF_LH2_ship[[d]][i] + EF_LH2_ship_add[[d]][i] + unloading_EF + EF_LH2_unloading_add[[d]][i]+
      regas_comp_EF + storage_H2g_EF
    
  }
  
  # Add the LH2 shipping emissions and totals to the database for this distance
  database[[paste0("EF_LH2_ship_", distance_ship[d])]] <- as.numeric(EF_LH2_ship[[d]])
  database[[paste0("EF_LH2_H2prod_add_", distance_ship[d])]] <- as.numeric(EF_LH2_H2prod_add[[d]])
  database[[paste0("EF_LH2_liqf_add_", distance_ship[d])]] <- as.numeric(EF_LH2_liqf_add[[d]])
  database[[paste0("EF_LH2_storage_add_", distance_ship[d])]] <- as.numeric(EF_LH2_storage_add[[d]])
  database[[paste0("EF_LH2_loading_add_", distance_ship[d])]] <- as.numeric(EF_LH2_loading_add[[d]])
  database[[paste0("EF_LH2_ship_add_", distance_ship[d])]] <- as.numeric(EF_LH2_ship_add[[d]]) 
  database[[paste0("EF_LH2_unloading_add_", distance_ship[d])]] <- as.numeric(EF_LH2_unloading_add[[d]])
  database[[paste0("EF_LH2_lifecycle_", distance_ship[d])]] <- as.numeric(EF_LH2_lifecycle[[d]])
  database[[paste0("EF_LH2_additions_", distance_ship[d])]] <- as.numeric(EF_LH2_additions[[d]]) 
  database[[paste0("EF_LH2_H2prod_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_H2prod_tot[[d]])
  database[[paste0("EF_LH2_liqf_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_liqf_tot[[d]])
  database[[paste0("EF_LH2_storage_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_storage_tot[[d]])
  database[[paste0("EF_LH2_loading_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_loading_tot[[d]])
  database[[paste0("EF_LH2_ship_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_ship_tot[[d]]) 
  database[[paste0("EF_LH2_unloading_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_unloading_tot[[d]])
  database[[paste0("EF_LH2_transport_", distance_ship[d])]] <- as.numeric(EF_LH2_transport[[d]])
  database[[paste0("EF_LH2_transport_tot_", distance_ship[d])]] <- as.numeric(EF_LH2_transport_tot[[d]]) 
}  



# Shipping transport via NH3. 

# Hydrogen requirement (kg H2 per kg NH3) and nitrogen requirement (kg N2 per kg NH3)
# based on molar ratio H2 and N2 vs NH3
ratio_H2_NH3 <- 2.016 * 1.5 / 17.031
ratio_N2_NH3 <- 28.0134 * 0.5 / 17.031


# We require a buffer of hydrogen to ensure continous operation of H-B synthesis:

# Storage days as buffer, only in the case of renewables (days)
H2g_buffer_ren_day <- 2

# Emissions associated with storage infra as buffer per kg H2 (gCO2e / kg H2)
H2g_buffer_ren_gCO2e <- E_H2g_storage * H2g_buffer_ren_day / 365 / vessel_kg_H2 / storage_H2g_life

# For excess, same storage as for intermittent renewables assumed
database$EF_NH3_H2g_buffer <- ifelse(database$electricity.type == "Grid", 0,
                                     ifelse(database$electricity.type == "Nuclear", 0,
                                            ifelse(database$elec.type.ren == "Hydropower", 0,
                                                   ifelse(database$electricity.type == "Grid (excess renewable)", H2g_buffer_ren_gCO2e,
                                                          ifelse(database$electricity.type == "Other/Unknown", H2g_buffer_ren_gCO2e,
                                                                 H2g_buffer_ren_gCO2e)))))

# We include two NH3 H-B synthesis options. Option 1 is fully electricity-based, option 2 also requires heat.
# in the analysis, we have only used option 1.
# Both include a compressor, so also a loss in compression. 

# Option 1 NH3
# Electricity requirement pre-compression (kWh / kg NH3)
P_precomp_1 <- 0.26 

# H-B + ASU electricity demand (kWh / kg NH3)
P_HB_1 <- 0.64

# Option 2 NH3
# Electricity requirement pre-compression (kWh / kg NH3)
P_precomp_2 <- 0.18

# H-B + ASU electricity demand (kWh / kg NH3)
P_HB_2 <- 0.56

# H-B heat demand (MJ / kg NH3)
Q_HB_2 <- 1.45

# For both NH3 production methods:
# Infrastructure emissions ASU (gCO2e / kg N2)
E_ASU <- 0.334

# Infrastructure emissions NH3 production (gCO2e / kg NH3)
E_NH3 <- 61.5

# Catalyst NH3 (gCO2e/ kg NH3)
E_cat <- 0.20

# Water input for reactor cooling (kg / kg NH3)
N_water_NH3 <- 1.5

# Loss ammonia synthesis (kg NH3 / kg NH3).
L_NH3 <- 5.5E-2

# Tanker shipping mass (kt NH3)
m_tanker_NH3 <- 109.248

# NH3 storage duration before shipping, max is 21 days (days).
# Only for size of storage tank, not for elec demand of recirculation of BOG, those are assumed to be included in loading elec demand
database$NH3_storage_loading_days <- ifelse(m_tanker_NH3 * ratio_H2_NH3/ (database$cap.ktH2_calc / 365) > 21, 21,
                                    m_tanker_NH3 * ratio_H2_NH3 / (database$cap.ktH2_calc / 365))


# We estimate the emissions associated with the storage vessel of liquid NH3 based on the hydrogen storage vessel.
# The vessel is 82 m3, could hold 527 kg H2 at 80 bar. L-NH3's density is 682.8 kg/m3
vessel_m3 <- 82
density_LNH3 <- 628.8
vessel_kg_NH3 <- vessel_m3 * density_LNH3

# Leakage NH3 during storage (kg NH3/ kg NH3)
L_NH3_storage <- 0.02E-2

# BOG during NH3 storage (kg NH3/ kg NH3 / day)
# Recirculated
BOG_NH3_storage <- 0.015E-2

# Loading electricity demand incl recirculation of BOG (kWh / kg NH3)
P_NH3_loading <- 0.08

# BOG during loading (kg NH3 / kg NH3)
BOG_NH3_loading <- 0.037E-2

# Unloading electricity demand incl recirculation of BOG (kWh / kg NH3)
P_NH3_unloading <- 0.08

# BOG during unloading (kg NH3/ kg NH3)
BOG_NH3_unloading <- 0.037E-2

# Upon arrival, two options.
# Option A: NH3 storage, incl leakage. (This is used in the Supplementary Information)

# Storage days at destination (days)
storage_NH3_dest_S_day <- 3

# Option B: cracking and storage as gaseous H2. (This is used in the main text)

# Storage days at destination (days)
storage_NH3_dest_C_day <- 1

# Electricity demand cracking (kWh / kg NH3)
P_NH3_cracking <- 0.4

# H2 recovery factor; rest is used for heat
H2_recovery <- 0.69

# Compression electricity after cracking (kWh / kg H2)
P_crack_comp <- 1.0


# Create a vector to store the NH3 production (method 1 and 2) emissions factors (gCO2e / kg H2)
EF_NH3_precomp_1 <- rep(0, nrow(database))
EF_NH3_precomp_2 <- rep(0, nrow(database))
EF_NH3_prod_1 <- rep(0, nrow(database))
EF_NH3_prod_2 <- rep(0, nrow(database))

# Create a vector to store the NH3 storage and loading/unloading emissions factors (gCO2e / kg H2)
# unloading_storage is for the case where at the destination, NH3 is stored, not cracked
EF_NH3_loading <- rep(0, nrow(database))
EF_NH3_unloading <- rep(0, nrow(database))
EF_NH3_unloading_storage <- rep(0, nrow(database))
EF_NH3_cracking <- rep(0, nrow(database))
EF_NH3_H2_comp <- rep(0, nrow(database))
EF_NH3_H2_storage <- rep(0, nrow(database))

  
  # Loop through each row of the database
  for (i in 1:nrow(database)) {
    # Get the local electricity footprint and NH3 storage days for this project, and H2 production footprint
    elec_CF <- database$elec_CF[i]
    storage_loading_days <- database$NH3_storage_loading_days[i]
    EF_hydrogen_buffer <- database$EF_NH3_H2g_buffer[i]
    f_utilisation <- database$utilisation.electrolyser[i]
    
    # Calculate the emissions factor for this row, making up for loss
    # Buffer of hydrogen only in case of renewable electricity.
    # Precompression of hydrogen is still per kg H2
    
    EF_NH3_precomp_1[i] <- (1 / (1 - L_comp)) * (P_precomp_1 * (elec_CF + E_comp_kWh / f_utilisation) + EF_hydrogen_buffer + L_comp * GWP_H2)
    
    EF_NH3_precomp_2[i] <- (1 / (1 - L_comp)) * (P_precomp_2 * (elec_CF + E_comp_kWh / f_utilisation) + EF_hydrogen_buffer + L_comp * GWP_H2)    
    
    # During NH3 synthesis only compression loss counted for GWP
    # molar ratio (ratio_H2_NH3) factor converts the output to per kg H2.
    # 0.18 kg H2 needed for 1 kg NH3. So production emissions for 1 kg NH3 need to be multiplied by (1/0.18) to correspond to 1 kg of H2
    # since NH3 does not have a GWP, this BOG-emission (GWP) is not included
    # For 1 kg NH3, the amount of N2 needed is given by ratio_N2_NH3. 

    EF_NH3_prod_1[i] <- (1/ratio_H2_NH3) * (1 / (1 - L_NH3)) * 
      (P_HB_1 * elec_CF + E_ASU * ratio_N2_NH3 + E_cat + E_NH3 + 
         N_water_NH3 * (P_desal * elec_CF + E_desal + E_deion)) 
      
    EF_NH3_prod_2[i] <-  (1/ratio_H2_NH3) * (1 / (1 - L_NH3)) * 
      (P_HB_2 * elec_CF + E_ASU * ratio_N2_NH3 + E_cat + E_NH3 + 
         N_water_NH3 * (P_desal * elec_CF + E_desal + E_deion))
    
    # NH3 liquefaction, storage facility based on hydrogen vessel, and loading.
    EF_NH3_loading[i] <- (1/ratio_H2_NH3) * (1 / (1 - BOG_NH3_loading)) * (P_NH3_loading * elec_CF + 
                                                                         E_H2g_storage * storage_loading_days / 365 / vessel_kg_NH3 / storage_H2g_life)

    # Unloading, re-liquefaction and storage. Storage 1 day when cracking, 3 days when sold as NH3.    
    EF_NH3_unloading[i] <- (1/ratio_H2_NH3) * (1 / (1 - BOG_NH3_unloading)) * (1 / (1 - L_NH3_storage)) * (P_NH3_unloading * renewable_avg_CF +
                                                                             E_H2g_storage * storage_NH3_dest_C_day / 365 / vessel_kg_NH3 / storage_H2g_life)
    
    EF_NH3_unloading_storage[i] <- (1/ratio_H2_NH3) * (1 / (1 - BOG_NH3_unloading)) * (1 / (1 - L_NH3_storage)) * (P_NH3_unloading * renewable_avg_CF +
                                                                                                                 E_H2g_storage * storage_NH3_dest_S_day / 365 / vessel_kg_NH3 / storage_H2g_life)
    
    # for cracking, part of the hydrogen is used for heat. It is then lost, but does not contribute to GWP
    EF_NH3_cracking[i] <- (1/ratio_H2_NH3) * (1 / H2_recovery) * P_NH3_cracking * renewable_avg_CF 
    
    # Compression infra emissions at destination: assumed that this facility can be used continuously so not multiplied by utilisation factor
    EF_NH3_H2_comp[i] <- (1 / (1 - L_comp)) * (P_crack_comp * (renewable_avg_CF + E_comp_kWh) + L_comp * GWP_H2)
    
    EF_NH3_H2_storage[i] <- H2g_dest_storage_gCO2e
  }

# Add the NH3 production emissions (method 1, 2), loading and unloading emissions to the database
database$EF_NH3_prod_1 <- as.numeric(EF_NH3_prod_1)
database$EF_NH3_prod_2 <- as.numeric(EF_NH3_prod_2)
database$EF_NH3_precomp_1 <- as.numeric(EF_NH3_precomp_1)
database$EF_NH3_precomp_2 <- as.numeric(EF_NH3_precomp_2)
database$EF_NH3_loading <- as.numeric(EF_NH3_loading)
database$EF_NH3_unloading <- as.numeric(EF_NH3_unloading)
database$EF_NH3_unloading_storage <- as.numeric(EF_NH3_unloading_storage)
database$EF_NH3_cracking <- as.numeric(EF_NH3_cracking)
database$EF_NH3_H2_comp <- as.numeric(EF_NH3_H2_comp)
database$EF_NH3_H2_storage <- as.numeric(EF_NH3_H2_storage)



# Shipping (ballast and laden voyage) incl fuel (gCO2e per kg*km)
E_NH3_ship <- (6.8361372 + 4.5376893) * 10^-3

# BOG NH3 shipping (kg NH3/ kg NH3 / day)
BOG_NH3_shipping <- 0.025E-2


# Create vectors to store the NH3 shipping emissions factors for each distance (gCO2e / kg H2)
EF_NH3_ship <- list()

# Create vectors to store the emissions from additional production as required later in the supply chain (gCO2e / kg H2)
EF_NH3_prod_H2_S_add <- list()
EF_NH3_precomp_1_S_add <- list()
EF_NH3_precomp_2_S_add <- list()
EF_NH3_prod_1_S_add <- list()
EF_NH3_prod_2_S_add <- list()
EF_NH3_loading_S_add <- list()
EF_NH3_ship_S_add <- list()

EF_NH3_prod_H2_C_add <- list()
EF_NH3_precomp_1_C_add <- list()
EF_NH3_precomp_2_C_add <- list()
EF_NH3_prod_1_C_add <- list()
EF_NH3_prod_2_C_add <- list()
EF_NH3_loading_C_add <- list()
EF_NH3_ship_C_add <- list()
EF_NH3_unloading_add <- list()
EF_NH3_crack_add <- list()

# Create vectors to store lifecycle emissions (production plus transport) for NH3 (gCO2e / kg H2)
EF_lifecycle_H2_NH3_1S <- list()
EF_lifecycle_H2_NH3_1C <- list()
EF_lifecycle_H2_NH3_2S <- list()
EF_lifecycle_H2_NH3_2C <- list()


EF_NH3_prod_H2_S_tot <- list()
EF_NH3_prod_H2_C_tot <- list()


# Create vectors to store all addded emissions (losses in other phases) (gCO2e / kg H2)
EF_NH3_additions_1S <- list()
EF_NH3_additions_2S <- list()
EF_NH3_additions_1C <- list()
EF_NH3_additions_2C <- list()

# Total emissions for everything after production, with and without downstream (gCO2e / kg H2)
EF_transport_NH3_1S <- list()
EF_transport_NH3_1S_tot <- list()
EF_transport_NH3_1C <- list()
EF_transport_NH3_1C_tot <- list()
 
# Loop through each distance
for (d in 1:length(distance_ship)) {
  
  # Create vectors to store data for this distance
  EF_NH3_ship[[d]] <- rep(0, nrow(database))
  
  EF_NH3_prod_H2_S_add[[d]]<- rep(0, nrow(database))
  EF_NH3_precomp_1_S_add[[d]]<- rep(0, nrow(database))
  EF_NH3_precomp_2_S_add[[d]]<- rep(0, nrow(database))
  EF_NH3_prod_1_S_add[[d]]<- rep(0, nrow(database))
  EF_NH3_prod_2_S_add[[d]]<- rep(0, nrow(database))
  EF_NH3_loading_S_add[[d]]<- rep(0, nrow(database))
  EF_NH3_ship_S_add[[d]]<- rep(0, nrow(database))
  
  EF_NH3_prod_H2_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_precomp_1_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_precomp_2_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_prod_1_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_prod_2_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_loading_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_ship_C_add[[d]]<- rep(0, nrow(database))
  EF_NH3_unloading_add[[d]]<- rep(0, nrow(database))
  EF_NH3_crack_add[[d]]<- rep(0, nrow(database))
  
  EF_lifecycle_H2_NH3_1S[[d]] <- rep(0, nrow(database))
  EF_lifecycle_H2_NH3_1C[[d]] <- rep(0, nrow(database)) 
  EF_lifecycle_H2_NH3_2S[[d]] <- rep(0, nrow(database))
  EF_lifecycle_H2_NH3_2C[[d]] <- rep(0, nrow(database))
  
  EF_NH3_prod_H2_S_tot[[d]] <- rep(0, nrow(database))
  EF_NH3_prod_H2_C_tot[[d]] <- rep(0, nrow(database))
  
  EF_NH3_additions_1S[[d]] <- rep(0, nrow(database))
  EF_NH3_additions_2S[[d]] <- rep(0, nrow(database))
  EF_NH3_additions_1C[[d]] <- rep(0, nrow(database))
  EF_NH3_additions_2C[[d]] <- rep(0, nrow(database))
  
  EF_transport_NH3_1S[[d]] <- rep(0, nrow(database))
  EF_transport_NH3_1S_tot[[d]] <- rep(0, nrow(database))
  EF_transport_NH3_1C[[d]] <- rep(0, nrow(database))
  EF_transport_NH3_1C_tot[[d]] <- rep(0, nrow(database))
  
  # Loop through each row of the database
  for (i in 1:nrow(database)) {
    
    # Get all the relevant inputs for this row
    elec_CF <- database$elec_CF[i]
    prod_H2_EF <- database$EF_production[i]
    H2_precomp_1 <- database$EF_NH3_precomp_1[i]
    H2_precomp_2 <- database$EF_NH3_precomp_2[i]
    prod_NH3_1 <- database$EF_NH3_prod_1[i]
    prod_NH3_2 <- database$EF_NH3_prod_2[i]
    NH3_load <- database$EF_NH3_loading[i]
    NH3_unload <- database$EF_NH3_unloading[i]
    NH3_unload_store <- database$EF_NH3_unloading_storage[i]
    NH3_crack <- database$EF_NH3_cracking[i]
    NH3_H2_comp <- database$EF_NH3_H2_comp[i]
    NH3_H2_store <- database$EF_NH3_H2_storage[i]
    
    
    # Calculate the emissions factor for this row and distance
    EF_NH3_ship[[d]][i] <- (1/ratio_H2_NH3) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * E_NH3_ship * distance_ship[d]

    EF_NH3_prod_H2_S_add[[d]][i] <- ((1 / (1 - L_comp)) * (1 / (1 - L_NH3)) * (1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * prod_H2_EF
    EF_NH3_precomp_1_S_add[[d]][i] <- ((1 / (1 - L_NH3)) * (1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * H2_precomp_1
    EF_NH3_precomp_2_S_add[[d]][i] <- ((1 / (1 - L_NH3)) * (1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * H2_precomp_2
    EF_NH3_prod_1_S_add[[d]][i] <- ((1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * prod_NH3_1
    EF_NH3_prod_2_S_add[[d]][i] <- ((1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * prod_NH3_2
    EF_NH3_loading_S_add[[d]][i] <- ((1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * NH3_load
    EF_NH3_ship_S_add[[d]][i] <- ((1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) - 1) * EF_NH3_ship[[d]][i]
    
    EF_NH3_prod_H2_C_add[[d]][i] <- ((1 / (1 - L_comp)) * (1 / (1 - L_NH3)) * (1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * prod_H2_EF
    EF_NH3_precomp_1_C_add[[d]][i] <- ((1 / (1 - L_NH3)) * (1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * H2_precomp_1
    EF_NH3_precomp_2_C_add[[d]][i] <- ((1 / (1 - L_NH3)) * (1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * H2_precomp_2
    EF_NH3_prod_1_C_add[[d]][i] <- ((1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * prod_NH3_1
    EF_NH3_prod_2_C_add[[d]][i] <- ((1 / (1 - BOG_NH3_loading)) * (1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * prod_NH3_2
    EF_NH3_loading_C_add[[d]][i] <- ((1 /(1 - BOG_NH3_shipping * distance_ship[d] / v_shipping)) * (1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * NH3_load
    EF_NH3_ship_C_add[[d]][i] <- ((1 / (1 - BOG_NH3_unloading)) * (1/ (1 - L_NH3_storage)) * (1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * EF_NH3_ship[[d]][i]  
    EF_NH3_unloading_add[[d]][i]  <- ((1 / H2_recovery) * (1 / (1 - L_comp)) - 1) * NH3_unload
    EF_NH3_crack_add[[d]][i] <- ((1 / (1 - L_comp)) - 1) * NH3_crack
    
    EF_lifecycle_H2_NH3_1S[[d]][i] <- prod_H2_EF + EF_NH3_prod_H2_S_add[[d]][i] + H2_precomp_1 + EF_NH3_precomp_1_S_add[[d]][i] +
      prod_NH3_1 + EF_NH3_prod_1_S_add[[d]][i] + NH3_load + EF_NH3_loading_S_add[[d]][i] + EF_NH3_ship[[d]][i] + EF_NH3_ship_S_add[[d]][i] +
      NH3_unload_store
    
    EF_lifecycle_H2_NH3_2S[[d]][i] <- prod_H2_EF + EF_NH3_prod_H2_S_add[[d]][i] + H2_precomp_2 + EF_NH3_precomp_2_S_add[[d]][i] +
      prod_NH3_2 + EF_NH3_prod_2_S_add[[d]][i] + NH3_load + EF_NH3_loading_S_add[[d]][i] + EF_NH3_ship[[d]][i] + EF_NH3_ship_S_add[[d]][i] +
      NH3_unload_store
    
    EF_lifecycle_H2_NH3_1C[[d]][i] <- prod_H2_EF + EF_NH3_prod_H2_C_add[[d]][i] + H2_precomp_1 + EF_NH3_precomp_1_C_add[[d]][i] +
      prod_NH3_1 + EF_NH3_prod_1_C_add[[d]][i] + NH3_load + EF_NH3_loading_C_add[[d]][i] + EF_NH3_ship[[d]][i] + EF_NH3_ship_C_add[[d]][i] +
      NH3_unload + EF_NH3_unloading_add[[d]][i] + NH3_crack + EF_NH3_crack_add[[d]][i] + NH3_H2_comp + NH3_H2_store
   
    EF_lifecycle_H2_NH3_2C[[d]][i] <- prod_H2_EF + EF_NH3_prod_H2_C_add[[d]][i] + H2_precomp_2 + EF_NH3_precomp_2_C_add[[d]][i] +
      prod_NH3_2 + EF_NH3_prod_2_C_add[[d]][i] + NH3_load + EF_NH3_loading_C_add[[d]][i] + EF_NH3_ship[[d]][i] + EF_NH3_ship_C_add[[d]][i] +
      NH3_unload + EF_NH3_unloading_add[[d]][i] + NH3_crack + EF_NH3_crack_add[[d]][i] + NH3_H2_comp + NH3_H2_store
  
    EF_NH3_additions_1S[[d]][i] <- EF_NH3_prod_H2_S_add[[d]][i] + EF_NH3_precomp_1_S_add[[d]][i] +
     EF_NH3_prod_1_S_add[[d]][i] + EF_NH3_loading_S_add[[d]][i] + EF_NH3_ship_S_add[[d]][i]
    
    EF_NH3_additions_2S[[d]][i] <- EF_NH3_prod_H2_S_add[[d]][i] + EF_NH3_precomp_2_S_add[[d]][i] +
      EF_NH3_prod_2_S_add[[d]][i] + EF_NH3_loading_S_add[[d]][i] + EF_NH3_ship_S_add[[d]][i]
    
    EF_NH3_additions_1C[[d]][i] <- EF_NH3_prod_H2_C_add[[d]][i] + EF_NH3_precomp_1_C_add[[d]][i] +
      EF_NH3_prod_1_C_add[[d]][i] + EF_NH3_loading_C_add[[d]][i] + EF_NH3_ship_C_add[[d]][i] +
     + EF_NH3_unloading_add[[d]][i]+ EF_NH3_crack_add[[d]][i]
    
    EF_NH3_additions_2C[[d]][i] <- EF_NH3_prod_H2_C_add[[d]][i] + EF_NH3_precomp_2_C_add[[d]][i] +
      EF_NH3_prod_2_C_add[[d]][i] + EF_NH3_loading_C_add[[d]][i] + EF_NH3_ship_C_add[[d]][i] +
      EF_NH3_unloading_add[[d]][i] + EF_NH3_crack_add[[d]][i]
    
  
    EF_NH3_prod_H2_S_tot[[d]][i] <- prod_H2_EF + EF_NH3_prod_H2_S_add[[d]][i]
    
    EF_NH3_prod_H2_C_tot[[d]][i] <- prod_H2_EF + EF_NH3_prod_H2_C_add[[d]][i]
    
    EF_transport_NH3_1S[[d]][i] <- H2_precomp_1 + prod_NH3_1 + NH3_load + EF_NH3_ship[[d]][i] + NH3_unload_store
    
    EF_transport_NH3_1S_tot[[d]][i] <- H2_precomp_1 + EF_NH3_precomp_1_S_add[[d]][i] +
      prod_NH3_1 + EF_NH3_prod_1_S_add[[d]][i] + NH3_load + EF_NH3_loading_S_add[[d]][i] + EF_NH3_ship[[d]][i] + EF_NH3_ship_S_add[[d]][i] +
      NH3_unload_store
    
    EF_transport_NH3_1C[[d]][i] <- H2_precomp_1 + prod_NH3_1 + NH3_load + EF_NH3_ship[[d]][i] + 
      NH3_unload + NH3_crack + NH3_H2_comp + NH3_H2_store
    
    EF_transport_NH3_1C_tot[[d]][i] <- H2_precomp_1 + EF_NH3_precomp_1_C_add[[d]][i] +
      prod_NH3_1 + EF_NH3_prod_1_C_add[[d]][i] + NH3_load + EF_NH3_loading_C_add[[d]][i] + EF_NH3_ship[[d]][i] + EF_NH3_ship_C_add[[d]][i] +
      NH3_unload + EF_NH3_unloading_add[[d]][i] + NH3_crack + EF_NH3_crack_add[[d]][i] + NH3_H2_comp + NH3_H2_store
    
    
    }
  
# Add the NH3 shipping emissions to the database for this distance
database[[paste0("EF_NH3_ship_", distance_ship[d])]] <- as.numeric(EF_NH3_ship[[d]])
database[[paste0("EF_NH3_prod_H2_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_H2_S_add[[d]])
database[[paste0("EF_NH3_precomp_1_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_precomp_1_S_add[[d]])
database[[paste0("EF_NH3_precomp_2_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_precomp_2_S_add[[d]])
database[[paste0("EF_NH3_prod_1_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_1_S_add[[d]])
database[[paste0("EF_NH3_prod_2_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_2_S_add[[d]])
database[[paste0("EF_NH3_loading_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_loading_S_add[[d]])
database[[paste0("EF_NH3_ship_S_add_", distance_ship[d])]] <- as.numeric(EF_NH3_ship_S_add[[d]])
database[[paste0("EF_NH3_prod_H2_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_H2_C_add[[d]])
database[[paste0("EF_NH3_precomp_1_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_precomp_1_C_add[[d]])
database[[paste0("EF_NH3_precomp_2_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_precomp_2_C_add[[d]])
database[[paste0("EF_NH3_prod_1_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_1_C_add[[d]])
database[[paste0("EF_NH3_prod_2_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_2_C_add[[d]])
database[[paste0("EF_NH3_loading_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_loading_C_add[[d]])
database[[paste0("EF_NH3_ship_C_add_", distance_ship[d])]] <- as.numeric(EF_NH3_ship_C_add[[d]])
database[[paste0("EF_NH3_unloading_add_", distance_ship[d])]] <- as.numeric(EF_NH3_unloading_add[[d]])
database[[paste0("EF_NH3_crack_add_", distance_ship[d])]] <- as.numeric(EF_NH3_crack_add[[d]])
database[[paste0("EF_lifecycle_H2_NH3_1S_", distance_ship[d])]] <- as.numeric(EF_lifecycle_H2_NH3_1S[[d]])
database[[paste0("EF_lifecycle_H2_NH3_2S_", distance_ship[d])]] <- as.numeric(EF_lifecycle_H2_NH3_2S[[d]])
database[[paste0("EF_lifecycle_H2_NH3_1C_", distance_ship[d])]] <- as.numeric(EF_lifecycle_H2_NH3_1C[[d]])
database[[paste0("EF_lifecycle_H2_NH3_2C_", distance_ship[d])]] <- as.numeric(EF_lifecycle_H2_NH3_2C[[d]])
database[[paste0("EF_NH3_prod_H2_S_tot_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_H2_S_tot[[d]]) 
database[[paste0("EF_NH3_prod_H2_C_tot_", distance_ship[d])]] <- as.numeric(EF_NH3_prod_H2_C_tot[[d]])
database[[paste0("EF_NH3_additions_1S_", distance_ship[d])]] <- as.numeric(EF_NH3_additions_1S[[d]])
database[[paste0("EF_NH3_additions_2S_", distance_ship[d])]] <- as.numeric(EF_NH3_additions_2S[[d]])
database[[paste0("EF_NH3_additions_1C_", distance_ship[d])]] <- as.numeric(EF_NH3_additions_1C[[d]])
database[[paste0("EF_NH3_additions_2C_", distance_ship[d])]] <- as.numeric(EF_NH3_additions_2C[[d]])
database[[paste0("EF_transport_NH3_1S_", distance_ship[d])]] <- as.numeric(EF_transport_NH3_1S[[d]])
database[[paste0("EF_transport_NH3_1S_tot_", distance_ship[d])]] <- as.numeric(EF_transport_NH3_1S_tot[[d]])
database[[paste0("EF_transport_NH3_1C_", distance_ship[d])]] <- as.numeric(EF_transport_NH3_1C[[d]])
database[[paste0("EF_transport_NH3_1C_tot_", distance_ship[d])]] <- as.numeric(EF_transport_NH3_1C_tot[[d]])


}  


# For visualisation purpose later
database$technology <- ifelse(database$technology == "Other Electrolysis", "Other", database$technology)


# Write output data for curtailment configuration to csv file, to be used to create figures
write.csv(database, "Derived_data/database_results_curt0.5.csv", row.names = FALSE)
